@@ -1,4 +1,10 @@
 // REQUIRES: nvptx-registered-target
+// RUN: %clang_cc1 -ffp-contract=off -triple nvptx-unknown-unknown -target-cpu sm_80 -target-feature +ptx70 \
+// RUN:            -fcuda-is-device -S -emit-llvm -o - -x cuda %s \
+// RUN:   | FileCheck -check-prefix=CHECK -check-prefix=CHECK_PTX70_SM80 -check-prefix=LP32 %s
+// RUN: %clang_cc1 -ffp-contract=off -triple nvptx64-unknown-unknown -target-cpu sm_80 -target-feature +ptx70 \
+// RUN:            -fcuda-is-device -S -emit-llvm -o - -x cuda %s \
+// RUN:   | FileCheck -check-prefix=CHECK -check-prefix=CHECK_PTX70_SM80 -check-prefix=LP64 %s
 // RUN: %clang_cc1 -ffp-contract=off -triple nvptx-unknown-unknown -target-cpu sm_60 \
 // RUN:            -fcuda-is-device -S -emit-llvm -o - -x cuda %s \
 // RUN:   | FileCheck -check-prefix=CHECK -check-prefix=LP32 %s
@@ -201,85 +207,85 @@ __shared__ long long sll;
 __device__ void nvvm_atom(float *fp, float f, double *dfp, double df, int *ip,
                           int i, unsigned int *uip, unsigned ui, long *lp,
                           long l, long long *llp, long long ll) {
-  // CHECK: atomicrmw add
+  // CHECK: atomicrmw add i32* {{.*}} seq_cst, align 4
   __nvvm_atom_add_gen_i(ip, i);
-  // CHECK: atomicrmw add
+  // CHECK: atomicrmw add i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_add_gen_l(&dl, l);
-  // CHECK: atomicrmw add
+  // CHECK: atomicrmw add i64* {{.*}} seq_cst, align 8
   __nvvm_atom_add_gen_ll(&sll, ll);
 
-  // CHECK: atomicrmw sub
+  // CHECK: atomicrmw sub i32* {{.*}} seq_cst, align 4
   __nvvm_atom_sub_gen_i(ip, i);
-  // CHECK: atomicrmw sub
+  // CHECK: atomicrmw sub i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_sub_gen_l(&dl, l);
-  // CHECK: atomicrmw sub
+  // CHECK: atomicrmw sub i64* {{.*}} seq_cst, align 8
   __nvvm_atom_sub_gen_ll(&sll, ll);
 
-  // CHECK: atomicrmw and
+  // CHECK: atomicrmw and i32* {{.*}} seq_cst, align 4
   __nvvm_atom_and_gen_i(ip, i);
-  // CHECK: atomicrmw and
+  // CHECK: atomicrmw and i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_and_gen_l(&dl, l);
-  // CHECK: atomicrmw and
+  // CHECK: atomicrmw and i64* {{.*}} seq_cst, align 8
   __nvvm_atom_and_gen_ll(&sll, ll);
 
-  // CHECK: atomicrmw or
+  // CHECK: atomicrmw or i32* {{.*}} seq_cst, align 4
   __nvvm_atom_or_gen_i(ip, i);
-  // CHECK: atomicrmw or
+  // CHECK: atomicrmw or i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_or_gen_l(&dl, l);
-  // CHECK: atomicrmw or
+  // CHECK: atomicrmw or i64* {{.*}} seq_cst, align 8
   __nvvm_atom_or_gen_ll(&sll, ll);
 
-  // CHECK: atomicrmw xor
+  // CHECK: atomicrmw xor i32* {{.*}} seq_cst, align 4
   __nvvm_atom_xor_gen_i(ip, i);
-  // CHECK: atomicrmw xor
+  // CHECK: atomicrmw xor i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_xor_gen_l(&dl, l);
-  // CHECK: atomicrmw xor
+  // CHECK: atomicrmw xor i64* {{.*}} seq_cst, align 8
   __nvvm_atom_xor_gen_ll(&sll, ll);
 
-  // CHECK: atomicrmw xchg
+  // CHECK: atomicrmw xchg i32* {{.*}} seq_cst, align 4
   __nvvm_atom_xchg_gen_i(ip, i);
-  // CHECK: atomicrmw xchg
+  // CHECK: atomicrmw xchg i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_xchg_gen_l(&dl, l);
-  // CHECK: atomicrmw xchg
+  // CHECK: atomicrmw xchg i64* {{.*}} seq_cst, align 8
   __nvvm_atom_xchg_gen_ll(&sll, ll);
 
-  // CHECK: atomicrmw max i32*
+  // CHECK: atomicrmw max i32* {{.*}} seq_cst, align 4
   __nvvm_atom_max_gen_i(ip, i);
-  // CHECK: atomicrmw umax i32*
+  // CHECK: atomicrmw umax i32* {{.*}} seq_cst, align 4
   __nvvm_atom_max_gen_ui((unsigned int *)ip, i);
-  // CHECK: atomicrmw max
+  // CHECK: atomicrmw max i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_max_gen_l(&dl, l);
-  // CHECK: atomicrmw umax
+  // CHECK: atomicrmw umax i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_max_gen_ul((unsigned long *)&dl, l);
-  // CHECK: atomicrmw max i64*
+  // CHECK: atomicrmw max i64* {{.*}} seq_cst, align 8
   __nvvm_atom_max_gen_ll(&sll, ll);
-  // CHECK: atomicrmw umax i64*
+  // CHECK: atomicrmw umax i64* {{.*}} seq_cst, align 8
   __nvvm_atom_max_gen_ull((unsigned long long *)&sll, ll);
 
-  // CHECK: atomicrmw min i32*
+  // CHECK: atomicrmw min i32* {{.*}} seq_cst, align 4
   __nvvm_atom_min_gen_i(ip, i);
-  // CHECK: atomicrmw umin i32*
+  // CHECK: atomicrmw umin i32* {{.*}} seq_cst, align 4
   __nvvm_atom_min_gen_ui((unsigned int *)ip, i);
-  // CHECK: atomicrmw min
+  // CHECK: atomicrmw min i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_min_gen_l(&dl, l);
-  // CHECK: atomicrmw umin
+  // CHECK: atomicrmw umin i{{32|64}}* {{.*}} seq_cst, align {{4|8}}
   __nvvm_atom_min_gen_ul((unsigned long *)&dl, l);
-  // CHECK: atomicrmw min i64*
+  // CHECK: atomicrmw min i64* {{.*}} seq_cst, align 8
   __nvvm_atom_min_gen_ll(&sll, ll);
-  // CHECK: atomicrmw umin i64*
+  // CHECK: atomicrmw umin i64* {{.*}} seq_cst, align 8
   __nvvm_atom_min_gen_ull((unsigned long long *)&sll, ll);
 
-  // CHECK: cmpxchg
+  // CHECK: cmpxchg i32* {{.*}} seq_cst seq_cst, align 4
   // CHECK-NEXT: extractvalue { i32, i1 } {{%[0-9]+}}, 0
   __nvvm_atom_cas_gen_i(ip, 0, i);
-  // CHECK: cmpxchg
+  // CHECK: cmpxchg i{{32|64}}* {{.*}} seq_cst seq_cst, align {{4|8}}
   // CHECK-NEXT: extractvalue { {{i32|i64}}, i1 } {{%[0-9]+}}, 0
   __nvvm_atom_cas_gen_l(&dl, 0, l);
-  // CHECK: cmpxchg
+  // CHECK: cmpxchg i64* {{.*}} seq_cst seq_cst, align 8
   // CHECK-NEXT: extractvalue { i64, i1 } {{%[0-9]+}}, 0
   __nvvm_atom_cas_gen_ll(&sll, 0, ll);
 
-  // CHECK: atomicrmw fadd float
+  // CHECK: atomicrmw fadd float* {{.*}} seq_cst, align 4
   __nvvm_atom_add_gen_f(fp, f);
 
   // CHECK: call i32 @llvm.nvvm.atomic.load.inc.32.p0i32
@@ -672,3 +678,80 @@ __device__ void nvvm_vote(int pred) {
   __nvvm_vote_ballot(pred);
   // CHECK: ret void
 }
+
+// CHECK-LABEL: nvvm_mbarrier
+__device__ void nvvm_mbarrier(long long* addr, __attribute__((address_space(3))) long long* sharedAddr, int count, long long state) {
+  #if __CUDA_ARCH__ >= 800
+  __nvvm_mbarrier_init(addr, count);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.mbarrier.init
+  __nvvm_mbarrier_init_shared(sharedAddr, count);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.mbarrier.init.shared
+
+  __nvvm_mbarrier_inval(addr);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.mbarrier.inval
+  __nvvm_mbarrier_inval_shared(sharedAddr);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.mbarrier.inval.shared
+
+  __nvvm_mbarrier_arrive(addr);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive
+  __nvvm_mbarrier_arrive_shared(sharedAddr);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.shared
+  __nvvm_mbarrier_arrive_noComplete(addr, count);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.noComplete
+  __nvvm_mbarrier_arrive_noComplete_shared(sharedAddr, count);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.noComplete.shared
+
+  __nvvm_mbarrier_arrive_drop(addr);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.drop
+  __nvvm_mbarrier_arrive_drop_shared(sharedAddr);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.drop.shared
+  __nvvm_mbarrier_arrive_drop_noComplete(addr, count);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.drop.noComplete
+  __nvvm_mbarrier_arrive_drop_noComplete_shared(sharedAddr, count);
+  // CHECK_PTX70_SM80: call i64 @llvm.nvvm.mbarrier.arrive.drop.noComplete.shared
+
+  __nvvm_mbarrier_test_wait(addr, state);
+  // CHECK_PTX70_SM80: call i1 @llvm.nvvm.mbarrier.test.wait
+  __nvvm_mbarrier_test_wait_shared(sharedAddr, state);
+  // CHECK_PTX70_SM80: call i1 @llvm.nvvm.mbarrier.test.wait.shared
+
+  __nvvm_mbarrier_pending_count(state);
+  // CHECK_PTX70_SM80: call i32 @llvm.nvvm.mbarrier.pending.count
+  #endif
+  // CHECK: ret void
+}
+
+// CHECK-LABEL: nvvm_async_copy
+__device__ void nvvm_async_copy(__attribute__((address_space(3))) void* dst, __attribute__((address_space(1))) const void* src, long long* addr, __attribute__((address_space(3))) long long* sharedAddr) {
+  #if __CUDA_ARCH__ >= 800
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.mbarrier.arrive
+  __nvvm_cp_async_mbarrier_arrive(addr);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.mbarrier.arrive.shared
+  __nvvm_cp_async_mbarrier_arrive_shared(sharedAddr);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.mbarrier.arrive.noinc
+  __nvvm_cp_async_mbarrier_arrive_noinc(addr);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.mbarrier.arrive.noinc.shared
+  __nvvm_cp_async_mbarrier_arrive_noinc_shared(sharedAddr);
+
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.ca.shared.global.4
+  __nvvm_cp_async_ca_shared_global_4(dst, src);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.ca.shared.global.8
+  __nvvm_cp_async_ca_shared_global_8(dst, src);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.ca.shared.global.16
+  __nvvm_cp_async_ca_shared_global_16(dst, src);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.cg.shared.global.16
+  __nvvm_cp_async_cg_shared_global_16(dst, src);
+
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.commit.group
+  __nvvm_cp_async_commit_group();
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.wait.group(i32 0)
+  __nvvm_cp_async_wait_group(0);
+    // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.wait.group(i32 8)
+  __nvvm_cp_async_wait_group(8);
+    // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.wait.group(i32 16)
+  __nvvm_cp_async_wait_group(16);
+  // CHECK_PTX70_SM80: call void @llvm.nvvm.cp.async.wait.all
+  __nvvm_cp_async_wait_all();
+  #endif
+  // CHECK: ret void
+}
\ No newline at end of file
