@@ -1,11 +1,17 @@
-// RUN: %clang_cc1 -emit-llvm -triple arm-none-eabi -o - %s | FileCheck %s
+// RUN: %clang_cc1 -emit-llvm -triple arm-none-eabi -o - %s | FileCheck %s --check-prefixes=CHECK,ELF
+// RUN: %clang_cc1 -emit-llvm -triple arm64-apple-ios -o - %s | FileCheck %s --check-prefixes=CHECK,MACHO
 // Test that global variables, statics and functions are attached section-attributes
 // as per '#pragma clang section' directives.
 
 extern "C" {
 // test with names for each section
+#ifdef __MACH__
+#pragma clang section bss = "__BSS,__mybss1" data = "__DATA,__mydata1" rodata = "__RODATA,__myrodata1"
+#pragma clang section text = "__TEXT,__mytext1"
+#else
 #pragma clang section bss="my_bss.1" data="my_data.1" rodata="my_rodata.1"
 #pragma clang section text="my_text.1"
+#endif
 int a;      // my_bss.1
 int b = 1;  // my_data.1
 int c[4];   // my_bss.1
@@ -19,7 +25,11 @@ int foo(void) {   // my_text.1
 static int g[2]; // my_bss.1
 #pragma clang section bss=""
 int h; // default - .bss
+#ifdef __MACH__
+#pragma clang section data = "" bss = "__BSS,__mybss2" text = "__TEXT,__mytext2"
+#else
 #pragma clang section data=""  bss="my_bss.2" text="my_text.2"
+#endif
 int i = 0; // my_bss.2
 extern const int j;
 const int j = 4; // default - .rodata
@@ -29,7 +39,11 @@ int goo(void) {  // my_text.2
   static int lstat_h;  // my_bss.2
   return zoo(g, &lstat_h);
 }
+#ifdef __MACH__
+#pragma clang section rodata = "__RODATA,__myrodata2" data = "__DATA,__mydata2" relro = "__RELRO,__myrelro2"
+#else
 #pragma clang section rodata="my_rodata.2" data="my_data.2" relro="my_relro.2"
+#endif
 int l = 5; // my_data.2
 extern const int m;
 const int m = 6; // my_rodata.2
@@ -65,19 +79,26 @@ int hoo(void) {
 //CHECK: @n ={{.*}} global i32 0, align 4
 //CHECK: @o ={{.*}} global i32 6, align 4
 //CHECK: @p ={{.*}} constant i32 7, align 4
-//CHECK: @_ZL5fptrs = internal constant [2 x i32 ()*] [i32 ()* @foo, i32 ()* @goo], align 4 #3
+//CHECK: @_ZL5fptrs = internal constant [2 x i32 ()*] [i32 ()* @foo, i32 ()* @goo], align {{4|8}} #3
 
 //CHECK: define{{.*}} i32 @foo() #5 {
 //CHECK: define{{.*}} i32 @goo() #6 {
 //CHECK: declare i32 @zoo(i32*, i32*) #7
 //CHECK: define{{.*}} i32 @hoo() #8 {
 
-//CHECK: attributes #0 = { "bss-section"="my_bss.1" "data-section"="my_data.1" "rodata-section"="my_rodata.1" }
-//CHECK: attributes #1 = { "data-section"="my_data.1" "rodata-section"="my_rodata.1" }
-//CHECK: attributes #2 = { "bss-section"="my_bss.2" "rodata-section"="my_rodata.1" }
-//CHECK: attributes #3 = { "bss-section"="my_bss.2" "data-section"="my_data.2" "relro-section"="my_relro.2" "rodata-section"="my_rodata.2" }
-//CHECK: attributes #4 = { "relro-section"="my_relro.2" }
-//CHECK: attributes #5 = { {{.*"implicit-section-name"="my_text.1".*}} }
-//CHECK: attributes #6 = { {{.*"implicit-section-name"="my_text.2".*}} }
+//ELF: attributes #0 = { "bss-section"="my_bss.1" "data-section"="my_data.1" "rodata-section"="my_rodata.1" }
+//ELF: attributes #1 = { "data-section"="my_data.1" "rodata-section"="my_rodata.1" }
+//ELF: attributes #2 = { "bss-section"="my_bss.2" "rodata-section"="my_rodata.1" }
+//ELF: attributes #3 = { "bss-section"="my_bss.2" "data-section"="my_data.2" "relro-section"="my_relro.2" "rodata-section"="my_rodata.2" }
+//ELF: attributes #4 = { "relro-section"="my_relro.2" }
+//ELF: attributes #5 = { {{.*"implicit-section-name"="my_text.1".*}} }
+//ELF: attributes #6 = { {{.*"implicit-section-name"="my_text.2".*}} }
+//MACHO: attributes #0 = { "bss-section"="__BSS,__mybss1" "data-section"="__DATA,__mydata1" "rodata-section"="__RODATA,__myrodata1" }
+//MACHO: attributes #1 = { "data-section"="__DATA,__mydata1" "rodata-section"="__RODATA,__myrodata1" }
+//MACHO: attributes #2 = { "bss-section"="__BSS,__mybss2" "rodata-section"="__RODATA,__myrodata1" }
+//MACHO: attributes #3 = { "bss-section"="__BSS,__mybss2" "data-section"="__DATA,__mydata2" "relro-section"="__RELRO,__myrelro2" "rodata-section"="__RODATA,__myrodata2" }
+//MACHO: attributes #4 = { "relro-section"="__RELRO,__myrelro2" }
+//MACHO: attributes #5 = { {{.*"implicit-section-name"="__TEXT,__mytext1".*}} }
+//MACHO: attributes #6 = { {{.*"implicit-section-name"="__TEXT,__mytext2".*}} }
 //CHECK-NOT: attributes #7 = { {{.*"implicit-section-name".*}} }
 //CHECK-NOT: attributes #8 = { {{.*"implicit-section-name".*}} }
