@@ -43,6 +43,6 @@ void RemQuoDiff(RemQuoFunc<T> func1, RemQuoFunc<T> func2, const uint8_t *data,
 
   __llvm_libc::fputil::FPBits<T> bits1(remainder1);
   __llvm_libc::fputil::FPBits<T> bits2(remainder2);
-  if (bits1.bitsAsUInt() != bits2.bitsAsUInt())
+  if (bits1.uintval() != bits2.uintval())
     __builtin_trap();
 }
