@@ -12,7 +12,7 @@
 
 namespace __llvm_libc {
 
-void LLVM_LIBC_ENTRYPOINT(bzero)(void *ptr, size_t count) {
+LLVM_LIBC_FUNCTION(void, bzero, (void *ptr, size_t count)) {
   GeneralPurposeMemset(reinterpret_cast<char *>(ptr), 0, count);
 }
 
