@@ -9,7 +9,7 @@
 #ifndef LLVM_LIBC_SRC_STRING_BZERO_H
 #define LLVM_LIBC_SRC_STRING_BZERO_H
 
-#include "include/string.h"
+#include <stddef.h> // size_t
 
 namespace __llvm_libc {
 
