@@ -16,10 +16,6 @@
 namespace __llvm_libc {
 namespace fputil {
 
-template <> struct MantissaWidth<long double> {
-  static constexpr unsigned value = 63;
-};
-
 template <unsigned Width> struct Padding;
 
 // i386 padding.
@@ -28,7 +24,7 @@ template <> struct Padding<4> { static constexpr unsigned value = 16; };
 // x86_64 padding.
 template <> struct Padding<8> { static constexpr unsigned value = 48; };
 
-template <> struct __attribute__((packed)) FPBits<long double> {
+template <> union FPBits<long double> {
   using UIntType = __uint128_t;
 
   static constexpr int exponentBias = 0x3FFF;
@@ -43,102 +39,133 @@ template <> struct __attribute__((packed)) FPBits<long double> {
       ((UIntType(maxExponent) - 1) << (MantissaWidth<long double>::value + 1)) |
       (UIntType(1) << MantissaWidth<long double>::value) | maxSubnormal;
 
-  UIntType mantissa : MantissaWidth<long double>::value;
-  uint8_t implicitBit : 1;
-  uint16_t exponent : ExponentWidth<long double>::value;
-  uint8_t sign : 1;
-  uint64_t padding : Padding<sizeof(uintptr_t)>::value;
+  using FloatProp = FloatProperties<long double>;
+
+  UIntType bits;
+
+  void setMantissa(UIntType mantVal) {
+    mantVal &= (FloatProp::mantissaMask);
+    bits &= ~(FloatProp::mantissaMask);
+    bits |= mantVal;
+  }
+
+  UIntType getMantissa() const { return bits & FloatProp::mantissaMask; }
+
+  void setUnbiasedExponent(UIntType expVal) {
+    expVal = (expVal << (FloatProp::bitWidth - 1 - FloatProp::exponentWidth)) &
+             FloatProp::exponentMask;
+    bits &= ~(FloatProp::exponentMask);
+    bits |= expVal;
+  }
+
+  uint16_t getUnbiasedExponent() const {
+    return uint16_t((bits & FloatProp::exponentMask) >>
+                    (FloatProp::bitWidth - 1 - FloatProp::exponentWidth));
+  }
+
+  void setImplicitBit(bool implicitVal) {
+    bits &= ~(UIntType(1) << FloatProp::mantissaWidth);
+    bits |= (UIntType(implicitVal) << FloatProp::mantissaWidth);
+  }
+
+  bool getImplicitBit() const {
+    return ((bits & (UIntType(1) << FloatProp::mantissaWidth)) >>
+            FloatProp::mantissaWidth);
+  }
+
+  void setSign(bool signVal) {
+    bits &= ~(FloatProp::signMask);
+    UIntType sign1 = UIntType(signVal) << (FloatProp::bitWidth - 1);
+    bits |= sign1;
+  }
+
+  bool getSign() const {
+    return ((bits & FloatProp::signMask) >> (FloatProp::bitWidth - 1));
+  }
+
+  long double val;
+
+  FPBits() : bits(0) {}
 
   template <typename XType,
             cpp::EnableIfType<cpp::IsSame<long double, XType>::Value, int> = 0>
-  explicit FPBits<long double>(XType x) {
-    *this = *reinterpret_cast<FPBits<long double> *>(&x);
+  explicit FPBits(XType x) : val(x) {}
+
+  template <typename XType,
+            cpp::EnableIfType<cpp::IsSame<XType, UIntType>::Value, int> = 0>
+  explicit FPBits(XType x) : bits(x) {}
+
+  operator long double() { return val; }
+
+  UIntType uintval() {
+    // We zero the padding bits as they can contain garbage.
+    static constexpr UIntType mask =
+        (UIntType(1) << (sizeof(long double) * 8 -
+                         Padding<sizeof(uintptr_t)>::value)) -
+        1;
+    return bits & mask;
   }
 
-  operator long double() { return *reinterpret_cast<long double *>(this); }
-
   int getExponent() const {
-    if (exponent == 0)
+    if (getUnbiasedExponent() == 0)
       return int(1) - exponentBias;
-    return int(exponent) - exponentBias;
+    return int(getUnbiasedExponent()) - exponentBias;
   }
 
   bool isZero() const {
-    return exponent == 0 && mantissa == 0 && implicitBit == 0;
+    return getUnbiasedExponent() == 0 && getMantissa() == 0 &&
+           getImplicitBit() == 0;
   }
 
   bool isInf() const {
-    return exponent == maxExponent && mantissa == 0 && implicitBit == 1;
+    return getUnbiasedExponent() == maxExponent && getMantissa() == 0 &&
+           getImplicitBit() == 1;
   }
 
   bool isNaN() const {
-    if (exponent == maxExponent) {
-      return (implicitBit == 0) || mantissa != 0;
-    } else if (exponent != 0) {
-      return implicitBit == 0;
+    if (getUnbiasedExponent() == maxExponent) {
+      return (getImplicitBit() == 0) || getMantissa() != 0;
+    } else if (getUnbiasedExponent() != 0) {
+      return getImplicitBit() == 0;
     }
     return false;
   }
 
   bool isInfOrNaN() const {
-    return (exponent == maxExponent) || (exponent != 0 && implicitBit == 0);
+    return (getUnbiasedExponent() == maxExponent) ||
+           (getUnbiasedExponent() != 0 && getImplicitBit() == 0);
   }
 
   // Methods below this are used by tests.
 
-  template <typename XType,
-            cpp::EnableIfType<cpp::IsSame<UIntType, XType>::Value, int> = 0>
-  explicit FPBits<long double>(XType x) {
-    // The last 4 bytes of v are ignored in case of i386.
-    *this = *reinterpret_cast<FPBits<long double> *>(&x);
-  }
-
-  UIntType bitsAsUInt() const {
-    // We cannot just return the bits as is as it will lead to reading
-    // out of bounds in case of i386. So, we first copy the wider value
-    // before returning the value. This makes the last 4 bytes are always
-    // zero in case i386.
-    UIntType result = UIntType(0);
-    *reinterpret_cast<FPBits<long double> *>(&result) = *this;
-
-    // Even though we zero out |result| before copying the long double value,
-    // there can be garbage bits in the padding. So, we zero the padding bits
-    // in |result|.
-    static constexpr UIntType mask =
-        (UIntType(1) << (sizeof(long double) * 8 -
-                         Padding<sizeof(uintptr_t)>::value)) -
-        1;
-    return result & mask;
-  }
-
   static FPBits<long double> zero() { return FPBits<long double>(0.0l); }
 
   static FPBits<long double> negZero() {
     FPBits<long double> bits(0.0l);
-    bits.sign = 1;
+    bits.setSign(1);
     return bits;
   }
 
   static FPBits<long double> inf() {
     FPBits<long double> bits(0.0l);
-    bits.exponent = maxExponent;
-    bits.implicitBit = 1;
+    bits.setUnbiasedExponent(maxExponent);
+    bits.setImplicitBit(1);
     return bits;
   }
 
   static FPBits<long double> negInf() {
     FPBits<long double> bits(0.0l);
-    bits.exponent = maxExponent;
-    bits.implicitBit = 1;
-    bits.sign = 1;
+    bits.setUnbiasedExponent(maxExponent);
+    bits.setImplicitBit(1);
+    bits.setSign(1);
     return bits;
   }
 
   static long double buildNaN(UIntType v) {
     FPBits<long double> bits(0.0l);
-    bits.exponent = maxExponent;
-    bits.implicitBit = 1;
-    bits.mantissa = v;
+    bits.setUnbiasedExponent(maxExponent);
+    bits.setImplicitBit(1);
+    bits.setMantissa(v);
     return bits;
   }
 };
