@@ -9,14 +9,13 @@
 #ifndef LLVM_LIBC_UTILS_FPUTIL_NEAREST_INTEGER_OPERATIONS_H
 #define LLVM_LIBC_UTILS_FPUTIL_NEAREST_INTEGER_OPERATIONS_H
 
-#include "FEnv.h"
+#include "FEnvUtils.h"
 #include "FPBits.h"
 
 #include "utils/CPP/TypeTraits.h"
 
 #include <math.h>
 #if math_errhandling & MATH_ERRNO
-#include "src/errno/llvmlibc_errno.h"
 #include <errno.h>
 #endif
 
@@ -44,15 +43,15 @@ static inline T trunc(T x) {
 
   // If the exponent is such that abs(x) is less than 1, then return 0.
   if (exponent <= -1) {
-    if (bits.sign)
+    if (bits.getSign())
       return T(-0.0);
     else
       return T(0.0);
   }
 
   int trimSize = MantissaWidth<T>::value - exponent;
-  bits.mantissa = (bits.mantissa >> trimSize) << trimSize;
-  return bits;
+  bits.setMantissa((bits.getMantissa() >> trimSize) << trimSize);
+  return T(bits);
 }
 
 template <typename T,
@@ -64,7 +63,7 @@ static inline T ceil(T x) {
   if (bits.isInfOrNaN() || bits.isZero())
     return x;
 
-  bool isNeg = bits.sign;
+  bool isNeg = bits.getSign();
   int exponent = bits.getExponent();
 
   // If the exponent is greater than the most negative mantissa
@@ -80,7 +79,7 @@ static inline T ceil(T x) {
   }
 
   uint32_t trimSize = MantissaWidth<T>::value - exponent;
-  bits.mantissa = (bits.mantissa >> trimSize) << trimSize;
+  bits.setMantissa((bits.getMantissa() >> trimSize) << trimSize);
   T truncValue = T(bits);
 
   // If x is already an integer, return it.
@@ -98,7 +97,7 @@ template <typename T,
           cpp::EnableIfType<cpp::IsFloatingPointType<T>::Value, int> = 0>
 static inline T floor(T x) {
   FPBits<T> bits(x);
-  if (bits.sign) {
+  if (bits.getSign()) {
     return -ceil(-x);
   } else {
     return trunc(x);
@@ -115,7 +114,7 @@ static inline T round(T x) {
   if (bits.isInfOrNaN() || bits.isZero())
     return x;
 
-  bool isNeg = bits.sign;
+  bool isNeg = bits.getSign();
   int exponent = bits.getExponent();
 
   // If the exponent is greater than the most negative mantissa
@@ -140,8 +139,8 @@ static inline T round(T x) {
   }
 
   uint32_t trimSize = MantissaWidth<T>::value - exponent;
-  bool halfBitSet = bits.mantissa & (UIntType(1) << (trimSize - 1));
-  bits.mantissa = (bits.mantissa >> trimSize) << trimSize;
+  bool halfBitSet = bits.getMantissa() & (UIntType(1) << (trimSize - 1));
+  bits.setMantissa((bits.getMantissa() >> trimSize) << trimSize);
   T truncValue = T(bits);
 
   // If x is already an integer, return it.
@@ -167,7 +166,7 @@ static inline T roundUsingCurrentRoundingMode(T x) {
   if (bits.isInfOrNaN() || bits.isZero())
     return x;
 
-  bool isNeg = bits.sign;
+  bool isNeg = bits.getSign();
   int exponent = bits.getExponent();
   int roundingMode = getRound();
 
@@ -185,7 +184,7 @@ static inline T roundUsingCurrentRoundingMode(T x) {
     case FE_TOWARDZERO:
       return isNeg ? T(-0.0) : T(0.0);
     case FE_TONEAREST:
-      if (exponent <= -2 || bits.mantissa == 0)
+      if (exponent <= -2 || bits.getMantissa() == 0)
         return isNeg ? T(-0.0) : T(0.0); // abs(x) <= 0.5
       else
         return isNeg ? T(-1.0) : T(1.0); // abs(x) > 0.5
@@ -196,19 +195,19 @@ static inline T roundUsingCurrentRoundingMode(T x) {
 
   uint32_t trimSize = MantissaWidth<T>::value - exponent;
   FPBits<T> newBits = bits;
-  newBits.mantissa = (bits.mantissa >> trimSize) << trimSize;
+  newBits.setMantissa((bits.getMantissa() >> trimSize) << trimSize);
   T truncValue = T(newBits);
 
   // If x is already an integer, return it.
   if (truncValue == x)
     return x;
 
-  UIntType trimValue = bits.mantissa & ((UIntType(1) << trimSize) - 1);
+  UIntType trimValue = bits.getMantissa() & ((UIntType(1) << trimSize) - 1);
   UIntType halfValue = (UIntType(1) << (trimSize - 1));
   // If exponent is 0, trimSize will be equal to the mantissa width, and
   // truncIsOdd` will not be correct. So, we handle it as a special case
   // below.
-  UIntType truncIsOdd = newBits.mantissa & (UIntType(1) << trimSize);
+  UIntType truncIsOdd = newBits.getMantissa() & (UIntType(1) << trimSize);
 
   switch (roundingMode) {
   case FE_DOWNWARD:
@@ -247,7 +246,7 @@ static inline I roundedFloatToSignedInteger(F x) {
   FPBits<F> bits(x);
   auto setDomainErrorAndRaiseInvalid = []() {
 #if math_errhandling & MATH_ERRNO
-    llvmlibc_errno = EDOM;
+    errno = EDOM; // NOLINT
 #endif
 #if math_errhandling & MATH_ERREXCEPT
     raiseExcept(FE_INVALID);
@@ -256,18 +255,18 @@ static inline I roundedFloatToSignedInteger(F x) {
 
   if (bits.isInfOrNaN()) {
     setDomainErrorAndRaiseInvalid();
-    return bits.sign ? IntegerMin : IntegerMax;
+    return bits.getSign() ? IntegerMin : IntegerMax;
   }
 
   int exponent = bits.getExponent();
   constexpr int exponentLimit = sizeof(I) * 8 - 1;
   if (exponent > exponentLimit) {
     setDomainErrorAndRaiseInvalid();
-    return bits.sign ? IntegerMin : IntegerMax;
+    return bits.getSign() ? IntegerMin : IntegerMax;
   } else if (exponent == exponentLimit) {
-    if (bits.sign == 0 || bits.mantissa != 0) {
+    if (bits.getSign() == 0 || bits.getMantissa() != 0) {
       setDomainErrorAndRaiseInvalid();
-      return bits.sign ? IntegerMin : IntegerMax;
+      return bits.getSign() ? IntegerMin : IntegerMax;
     }
     // If the control reaches here, then it means that the rounded
     // value is the most negative number for the signed integer type I.
