@@ -24,7 +24,7 @@
 int main(int, char**)
 {
     {
-    typedef test_compare<std::less<int> > C;
+    typedef test_less<int> C;
     const std::map<int, double, C> m(C(3));
     assert(m.empty());
     assert(m.begin() == m.end());
@@ -32,7 +32,7 @@ int main(int, char**)
     }
 #if TEST_STD_VER >= 11
     {
-    typedef test_compare<std::less<int> > C;
+    typedef test_less<int> C;
     const std::map<int, double, C, min_allocator<std::pair<const int, double>>> m(C(3));
     assert(m.empty());
     assert(m.begin() == m.end());
