@@ -6,8 +6,6 @@
 //
 //===----------------------------------------------------------------------===//
 
-// UNSUPPORTED: c++03
-
 // <fstream>
 
 // template <class charT, class traits = char_traits<charT> >
@@ -31,7 +29,7 @@ int main(int, char**)
         assert(f.sputn("123", 3) == 3);
         f.pubseekoff(1, std::ios_base::beg);
         assert(f.sgetc() == '2');
-        std::filebuf f2(move(f));
+        std::filebuf f2(std::move(f));
         assert(!f.is_open());
         assert(f2.is_open());
         assert(f2.sgetc() == '2');
@@ -45,7 +43,7 @@ int main(int, char**)
         assert(f.sputn(L"123", 3) == 3);
         f.pubseekoff(1, std::ios_base::beg);
         assert(f.sgetc() == L'2');
-        std::wfilebuf f2(move(f));
+        std::wfilebuf f2(std::move(f));
         assert(!f.is_open());
         assert(f2.is_open());
         assert(f2.sgetc() == L'2');
