@@ -78,15 +78,24 @@ TEST_CASE(static_env_test)
 TEST_CASE(test_is_directory_fails)
 {
     scoped_test_env env;
+#ifdef _WIN32
+    // Windows doesn't support setting perms::none to trigger failures
+    // reading directories; test using a special inaccessible directory
+    // instead.
+    const path p = GetWindowsInaccessibleDir();
+    if (p.empty())
+        TEST_UNSUPPORTED();
+#else
     const path dir = env.create_dir("dir");
-    const path dir2 = env.create_dir("dir/dir2");
+    const path p = env.create_dir("dir/dir2");
     permissions(dir, perms::none);
+#endif
 
     std::error_code ec;
-    TEST_CHECK(is_directory(dir2, ec) == false);
+    TEST_CHECK(is_directory(p, ec) == false);
     TEST_CHECK(ec);
 
-    TEST_CHECK_THROW(filesystem_error, is_directory(dir2));
+    TEST_CHECK_THROW(filesystem_error, is_directory(p));
 }
 
 TEST_SUITE_END()
