@@ -10,9 +10,7 @@
 // UNSUPPORTED: c++03, c++11
 
 // dylib support for shared_mutex was added in macosx10.12
-// XFAIL: with_system_cxx_lib=macosx10.11
-// XFAIL: with_system_cxx_lib=macosx10.10
-// XFAIL: with_system_cxx_lib=macosx10.9
+// XFAIL: use_system_cxx_lib && target={{.+}}-apple-macosx10.{{9|10|11}}
 
 // <shared_mutex>
 
@@ -22,6 +20,7 @@
 
 #include <shared_mutex>
 #include <cassert>
+#include <mutex>
 #include "nasty_containers.h"
 
 #include "test_macros.h"
