@@ -9,6 +9,7 @@
 #ifndef LLD_COFF_ICF_H
 #define LLD_COFF_ICF_H
 
+#include "Config.h"
 #include "lld/Common/LLVM.h"
 #include "llvm/ADT/ArrayRef.h"
 
@@ -17,7 +18,7 @@ namespace coff {
 
 class Chunk;
 
-void doICF(ArrayRef<Chunk *> chunks);
+void doICF(ArrayRef<Chunk *> chunks, ICFLevel);
 
 } // namespace coff
 } // namespace lld
