@@ -13,6 +13,7 @@
 #include "lld/Common/ErrorHandler.h"
 #include "lld/Common/LLVM.h"
 #include "llvm/Support/LEB128.h"
+#include "llvm/Support/xxhash.h"
 
 #define DEBUG_TYPE "lld"
 
@@ -38,6 +39,11 @@ bool relocIs64(uint8_t relocType) {
   case R_WASM_MEMORY_ADDR_SLEB64:
   case R_WASM_MEMORY_ADDR_REL_SLEB64:
   case R_WASM_MEMORY_ADDR_I64:
+  case R_WASM_TABLE_INDEX_SLEB64:
+  case R_WASM_TABLE_INDEX_I64:
+  case R_WASM_FUNCTION_OFFSET_I64:
+  case R_WASM_TABLE_INDEX_REL_SLEB64:
+  case R_WASM_MEMORY_ADDR_TLS_SLEB64:
     return true;
   default:
     return false;
@@ -56,102 +62,67 @@ StringRef InputChunk::getComdatName() const {
   return file->getWasmObj()->linkingData().Comdats[index];
 }
 
-void InputChunk::verifyRelocTargets() const {
-  for (const WasmRelocation &rel : relocations) {
-    uint64_t existingValue;
-    unsigned bytesRead = 0;
-    unsigned paddedLEBWidth = 5;
-    auto offset = rel.Offset - getInputSectionOffset();
-    const uint8_t *loc = data().data() + offset;
-    switch (rel.Type) {
-    case R_WASM_TYPE_INDEX_LEB:
-    case R_WASM_FUNCTION_INDEX_LEB:
-    case R_WASM_GLOBAL_INDEX_LEB:
-    case R_WASM_EVENT_INDEX_LEB:
-    case R_WASM_MEMORY_ADDR_LEB:
-    case R_WASM_TABLE_NUMBER_LEB:
-      existingValue = decodeULEB128(loc, &bytesRead);
-      break;
-    case R_WASM_MEMORY_ADDR_LEB64:
-      existingValue = decodeULEB128(loc, &bytesRead);
-      paddedLEBWidth = 10;
-      break;
-    case R_WASM_TABLE_INDEX_SLEB:
-    case R_WASM_TABLE_INDEX_REL_SLEB:
-    case R_WASM_MEMORY_ADDR_SLEB:
-    case R_WASM_MEMORY_ADDR_REL_SLEB:
-    case R_WASM_MEMORY_ADDR_TLS_SLEB:
-      existingValue = static_cast<uint64_t>(decodeSLEB128(loc, &bytesRead));
-      break;
-    case R_WASM_TABLE_INDEX_SLEB64:
-    case R_WASM_MEMORY_ADDR_SLEB64:
-    case R_WASM_MEMORY_ADDR_REL_SLEB64:
-      existingValue = static_cast<uint64_t>(decodeSLEB128(loc, &bytesRead));
-      paddedLEBWidth = 10;
-      break;
-    case R_WASM_TABLE_INDEX_I32:
-    case R_WASM_MEMORY_ADDR_I32:
-    case R_WASM_FUNCTION_OFFSET_I32:
-    case R_WASM_SECTION_OFFSET_I32:
-    case R_WASM_GLOBAL_INDEX_I32:
-      existingValue = read32le(loc);
-      break;
-    case R_WASM_TABLE_INDEX_I64:
-    case R_WASM_MEMORY_ADDR_I64:
-    case R_WASM_FUNCTION_OFFSET_I64:
-      existingValue = read64le(loc);
-      break;
-    default:
-      llvm_unreachable("unknown relocation type");
-    }
+uint32_t InputChunk::getSize() const {
+  if (const auto *ms = dyn_cast<SyntheticMergedChunk>(this))
+    return ms->builder.getSize();
 
-    if (bytesRead && bytesRead != paddedLEBWidth)
-      warn("expected LEB at relocation site be 5/10-byte padded");
-
-    if (rel.Type != R_WASM_GLOBAL_INDEX_LEB &&
-        rel.Type != R_WASM_GLOBAL_INDEX_I32) {
-      auto expectedValue = file->calcExpectedValue(rel);
-      if (expectedValue != existingValue)
-        warn(toString(this) + ": unexpected existing value for " +
-             relocTypeToString(rel.Type) + ": existing=" +
-             Twine(existingValue) + " expected=" + Twine(expectedValue));
+  if (const auto *f = dyn_cast<InputFunction>(this)) {
+    if (config->compressRelocations && f->file) {
+      return f->getCompressedSize();
     }
   }
+
+  return data().size();
+}
+
+uint32_t InputChunk::getInputSize() const {
+  if (const auto *f = dyn_cast<InputFunction>(this))
+    return f->function->Size;
+  return getSize();
 }
 
 // Copy this input chunk to an mmap'ed output file and apply relocations.
 void InputChunk::writeTo(uint8_t *buf) const {
+  if (const auto *f = dyn_cast<InputFunction>(this)) {
+    if (file && config->compressRelocations)
+      return f->writeCompressed(buf);
+  } else if (const auto *ms = dyn_cast<SyntheticMergedChunk>(this)) {
+    ms->builder.write(buf + outSecOff);
+    // Apply relocations
+    ms->relocate(buf + outSecOff);
+    return;
+  }
+
   // Copy contents
-  memcpy(buf + outputOffset, data().data(), data().size());
+  memcpy(buf + outSecOff, data().data(), data().size());
 
   // Apply relocations
+  relocate(buf + outSecOff);
+}
+
+void InputChunk::relocate(uint8_t *buf) const {
   if (relocations.empty())
     return;
 
-#ifndef NDEBUG
-  verifyRelocTargets();
-#endif
-
   LLVM_DEBUG(dbgs() << "applying relocations: " << toString(this)
                     << " count=" << relocations.size() << "\n");
-  int32_t off = outputOffset - getInputSectionOffset();
-  auto tombstone = getTombstone();
+  int32_t inputSectionOffset = getInputSectionOffset();
+  uint64_t tombstone = getTombstone();
 
   for (const WasmRelocation &rel : relocations) {
-    uint8_t *loc = buf + rel.Offset + off;
-    auto value = file->calcNewValue(rel, tombstone);
+    uint8_t *loc = buf + rel.Offset - inputSectionOffset;
     LLVM_DEBUG(dbgs() << "apply reloc: type=" << relocTypeToString(rel.Type));
     if (rel.Type != R_WASM_TYPE_INDEX_LEB)
       LLVM_DEBUG(dbgs() << " sym=" << file->getSymbols()[rel.Index]->getName());
     LLVM_DEBUG(dbgs() << " addend=" << rel.Addend << " index=" << rel.Index
-                      << " value=" << value << " offset=" << rel.Offset
-                      << "\n");
+                      << " offset=" << rel.Offset << "\n");
+    auto value = file->calcNewValue(rel, tombstone, this);
 
     switch (rel.Type) {
     case R_WASM_TYPE_INDEX_LEB:
     case R_WASM_FUNCTION_INDEX_LEB:
     case R_WASM_GLOBAL_INDEX_LEB:
-    case R_WASM_EVENT_INDEX_LEB:
+    case R_WASM_TAG_INDEX_LEB:
     case R_WASM_MEMORY_ADDR_LEB:
     case R_WASM_TABLE_NUMBER_LEB:
       encodeULEB128(value, loc, 5);
@@ -167,8 +138,10 @@ void InputChunk::writeTo(uint8_t *buf) const {
       encodeSLEB128(static_cast<int32_t>(value), loc, 5);
       break;
     case R_WASM_TABLE_INDEX_SLEB64:
+    case R_WASM_TABLE_INDEX_REL_SLEB64:
     case R_WASM_MEMORY_ADDR_SLEB64:
     case R_WASM_MEMORY_ADDR_REL_SLEB64:
+    case R_WASM_MEMORY_ADDR_TLS_SLEB64:
       encodeSLEB128(static_cast<int64_t>(value), loc, 10);
       break;
     case R_WASM_TABLE_INDEX_I32:
@@ -176,6 +149,7 @@ void InputChunk::writeTo(uint8_t *buf) const {
     case R_WASM_FUNCTION_OFFSET_I32:
     case R_WASM_SECTION_OFFSET_I32:
     case R_WASM_GLOBAL_INDEX_I32:
+    case R_WASM_MEMORY_ADDR_LOCREL_I32:
       write32le(loc, value);
       break;
     case R_WASM_TABLE_INDEX_I64:
@@ -196,7 +170,7 @@ void InputChunk::writeRelocations(raw_ostream &os) const {
   if (relocations.empty())
     return;
 
-  int32_t off = outputOffset - getInputSectionOffset();
+  int32_t off = outSecOff - getInputSectionOffset();
   LLVM_DEBUG(dbgs() << "writeRelocations: " << file->getName()
                     << " offset=" << Twine(off) << "\n");
 
@@ -210,6 +184,14 @@ void InputChunk::writeRelocations(raw_ostream &os) const {
   }
 }
 
+uint64_t InputChunk::getTombstone() const {
+  if (const auto *s = dyn_cast<InputSection>(this)) {
+    return s->tombstoneValue;
+  }
+
+  return 0;
+}
+
 void InputFunction::setFunctionIndex(uint32_t index) {
   LLVM_DEBUG(dbgs() << "InputFunction::setFunctionIndex: " << getName()
                     << " -> " << index << "\n");
@@ -232,7 +214,7 @@ static unsigned writeCompressedReloc(uint8_t *buf, const WasmRelocation &rel,
   case R_WASM_TYPE_INDEX_LEB:
   case R_WASM_FUNCTION_INDEX_LEB:
   case R_WASM_GLOBAL_INDEX_LEB:
-  case R_WASM_EVENT_INDEX_LEB:
+  case R_WASM_TAG_INDEX_LEB:
   case R_WASM_MEMORY_ADDR_LEB:
   case R_WASM_MEMORY_ADDR_LEB64:
   case R_WASM_TABLE_NUMBER_LEB:
@@ -252,7 +234,7 @@ static unsigned getRelocWidthPadded(const WasmRelocation &rel) {
   case R_WASM_TYPE_INDEX_LEB:
   case R_WASM_FUNCTION_INDEX_LEB:
   case R_WASM_GLOBAL_INDEX_LEB:
-  case R_WASM_EVENT_INDEX_LEB:
+  case R_WASM_TAG_INDEX_LEB:
   case R_WASM_MEMORY_ADDR_LEB:
   case R_WASM_TABLE_NUMBER_LEB:
   case R_WASM_TABLE_INDEX_SLEB:
@@ -296,13 +278,14 @@ void InputFunction::calculateSize() {
   uint32_t start = getInputSectionOffset();
   uint32_t end = start + function->Size;
 
-  auto tombstone = getTombstone();
+  uint64_t tombstone = getTombstone();
 
   uint32_t lastRelocEnd = start + functionSizeLength;
   for (const WasmRelocation &rel : relocations) {
     LLVM_DEBUG(dbgs() << "  region: " << (rel.Offset - lastRelocEnd) << "\n");
     compressedFuncSize += rel.Offset - lastRelocEnd;
-    compressedFuncSize += getRelocWidth(rel, file->calcNewValue(rel, tombstone));
+    compressedFuncSize +=
+        getRelocWidth(rel, file->calcNewValue(rel, tombstone, this));
     lastRelocEnd = rel.Offset + getRelocWidthPadded(rel);
   }
   LLVM_DEBUG(dbgs() << "  final region: " << (end - lastRelocEnd) << "\n");
@@ -319,18 +302,15 @@ void InputFunction::calculateSize() {
 
 // Override the default writeTo method so that we can (optionally) write the
 // compressed version of the function.
-void InputFunction::writeTo(uint8_t *buf) const {
-  if (!file || !config->compressRelocations)
-    return InputChunk::writeTo(buf);
-
-  buf += outputOffset;
+void InputFunction::writeCompressed(uint8_t *buf) const {
+  buf += outSecOff;
   uint8_t *orig = buf;
   (void)orig;
 
   const uint8_t *secStart = file->codeSection->Content.data();
   const uint8_t *funcStart = secStart + getInputSectionOffset();
   const uint8_t *end = funcStart + function->Size;
-  auto tombstone = getTombstone();
+  uint64_t tombstone = getTombstone();
   uint32_t count;
   decodeULEB128(funcStart, &count);
   funcStart += count;
@@ -343,7 +323,8 @@ void InputFunction::writeTo(uint8_t *buf) const {
     LLVM_DEBUG(dbgs() << "  write chunk: " << chunkSize << "\n");
     memcpy(buf, lastRelocEnd, chunkSize);
     buf += chunkSize;
-    buf += writeCompressedReloc(buf, rel, file->calcNewValue(rel, tombstone));
+    buf += writeCompressedReloc(buf, rel,
+                                file->calcNewValue(rel, tombstone, this));
     lastRelocEnd = secStart + rel.Offset + getRelocWidthPadded(rel);
   }
 
@@ -353,31 +334,48 @@ void InputFunction::writeTo(uint8_t *buf) const {
   LLVM_DEBUG(dbgs() << "  total: " << (buf + chunkSize - orig) << "\n");
 }
 
+uint64_t InputChunk::getChunkOffset(uint64_t offset) const {
+  if (const auto *ms = dyn_cast<MergeInputChunk>(this)) {
+    LLVM_DEBUG(dbgs() << "getChunkOffset(merged): " << getName() << "\n");
+    LLVM_DEBUG(dbgs() << "offset: " << offset << "\n");
+    LLVM_DEBUG(dbgs() << "parentOffset: " << ms->getParentOffset(offset)
+                      << "\n");
+    assert(ms->parent);
+    return ms->parent->getChunkOffset(ms->getParentOffset(offset));
+  }
+  return outputSegmentOffset + offset;
+}
+
+uint64_t InputChunk::getOffset(uint64_t offset) const {
+  return outSecOff + getChunkOffset(offset);
+}
+
+uint64_t InputChunk::getVA(uint64_t offset) const {
+  return (outputSeg ? outputSeg->startVA : 0) + getChunkOffset(offset);
+}
+
 // Generate code to apply relocations to the data section at runtime.
 // This is only called when generating shared libaries (PIC) where address are
 // not known at static link time.
-void InputSegment::generateRelocationCode(raw_ostream &os) const {
+void InputChunk::generateRelocationCode(raw_ostream &os) const {
   LLVM_DEBUG(dbgs() << "generating runtime relocations: " << getName()
                     << " count=" << relocations.size() << "\n");
 
-  unsigned opcode_ptr_const = config->is64.getValueOr(false)
-                                  ? WASM_OPCODE_I64_CONST
-                                  : WASM_OPCODE_I32_CONST;
-  unsigned opcode_ptr_add = config->is64.getValueOr(false)
-                                ? WASM_OPCODE_I64_ADD
-                                : WASM_OPCODE_I32_ADD;
+  bool is64 = config->is64.getValueOr(false);
+  unsigned opcode_ptr_const = is64 ? WASM_OPCODE_I64_CONST
+                                   : WASM_OPCODE_I32_CONST;
+  unsigned opcode_ptr_add = is64 ? WASM_OPCODE_I64_ADD
+                                 : WASM_OPCODE_I32_ADD;
 
-  auto tombstone = getTombstone();
+  uint64_t tombstone = getTombstone();
   // TODO(sbc): Encode the relocations in the data section and write a loop
   // here to apply them.
-  uint64_t segmentVA = outputSeg->startVA + outputSegmentOffset;
   for (const WasmRelocation &rel : relocations) {
-    uint64_t offset = rel.Offset - getInputSectionOffset();
-    uint64_t outputOffset = segmentVA + offset;
+    uint64_t offset = getVA(rel.Offset) - getInputSectionOffset();
 
     LLVM_DEBUG(dbgs() << "gen reloc: type=" << relocTypeToString(rel.Type)
                       << " addend=" << rel.Addend << " index=" << rel.Index
-                      << " output offset=" << outputOffset << "\n");
+                      << " output offset=" << offset << "\n");
 
     // Get __memory_base
     writeU8(os, WASM_OPCODE_GLOBAL_GET, "GLOBAL_GET");
@@ -385,7 +383,7 @@ void InputSegment::generateRelocationCode(raw_ostream &os) const {
 
     // Add the offset of the relocation
     writeU8(os, opcode_ptr_const, "CONST");
-    writeSleb128(os, outputOffset, "offset");
+    writeSleb128(os, offset, "offset");
     writeU8(os, opcode_ptr_add, "ADD");
 
     bool is64 = relocIs64(rel.Type);
@@ -414,7 +412,7 @@ void InputSegment::generateRelocationCode(raw_ostream &os) const {
       writeU8(os, WASM_OPCODE_GLOBAL_GET, "GLOBAL_GET");
       writeUleb128(os, baseSymbol->getGlobalIndex(), "base");
       writeU8(os, opcode_reloc_const, "CONST");
-      writeSleb128(os, file->calcNewValue(rel, tombstone), "offset");
+      writeSleb128(os, file->calcNewValue(rel, tombstone, this), "offset");
       writeU8(os, opcode_reloc_add, "ADD");
     }
 
@@ -425,6 +423,82 @@ void InputSegment::generateRelocationCode(raw_ostream &os) const {
   }
 }
 
+// Split WASM_SEG_FLAG_STRINGS section. Such a section is a sequence of
+// null-terminated strings.
+void MergeInputChunk::splitStrings(ArrayRef<uint8_t> data) {
+  LLVM_DEBUG(llvm::dbgs() << "splitStrings\n");
+  size_t off = 0;
+  StringRef s = toStringRef(data);
+
+  while (!s.empty()) {
+    size_t end = s.find(0);
+    if (end == StringRef::npos)
+      fatal(toString(this) + ": string is not null terminated");
+    size_t size = end + 1;
+
+    pieces.emplace_back(off, xxHash64(s.substr(0, size)), true);
+    s = s.substr(size);
+    off += size;
+  }
+}
+
+// This function is called after we obtain a complete list of input sections
+// that need to be linked. This is responsible to split section contents
+// into small chunks for further processing.
+//
+// Note that this function is called from parallelForEach. This must be
+// thread-safe (i.e. no memory allocation from the pools).
+void MergeInputChunk::splitIntoPieces() {
+  assert(pieces.empty());
+  // As of now we only support WASM_SEG_FLAG_STRINGS but in the future we
+  // could add other types of splitting (see ELF's splitIntoPieces).
+  assert(flags & WASM_SEG_FLAG_STRINGS);
+  splitStrings(data());
+}
+
+SectionPiece *MergeInputChunk::getSectionPiece(uint64_t offset) {
+  if (this->data().size() <= offset)
+    fatal(toString(this) + ": offset is outside the section");
+
+  // If Offset is not at beginning of a section piece, it is not in the map.
+  // In that case we need to  do a binary search of the original section piece
+  // vector.
+  auto it = partition_point(
+      pieces, [=](SectionPiece p) { return p.inputOff <= offset; });
+  return &it[-1];
+}
+
+// Returns the offset in an output section for a given input offset.
+// Because contents of a mergeable section is not contiguous in output,
+// it is not just an addition to a base output offset.
+uint64_t MergeInputChunk::getParentOffset(uint64_t offset) const {
+  // If Offset is not at beginning of a section piece, it is not in the map.
+  // In that case we need to search from the original section piece vector.
+  const SectionPiece *piece = getSectionPiece(offset);
+  uint64_t addend = offset - piece->inputOff;
+  return piece->outputOff + addend;
+}
+
+void SyntheticMergedChunk::finalizeContents() {
+  // Add all string pieces to the string table builder to create section
+  // contents.
+  for (MergeInputChunk *sec : chunks)
+    for (size_t i = 0, e = sec->pieces.size(); i != e; ++i)
+      if (sec->pieces[i].live)
+        builder.add(sec->getData(i));
+
+  // Fix the string table content. After this, the contents will never change.
+  builder.finalize();
+
+  // finalize() fixed tail-optimized strings, so we can now get
+  // offsets of strings. Get an offset for each string and save it
+  // to a corresponding SectionPiece for easy access.
+  for (MergeInputChunk *sec : chunks)
+    for (size_t i = 0, e = sec->pieces.size(); i != e; ++i)
+      if (sec->pieces[i].live)
+        sec->pieces[i].outputOff = builder.getOffset(sec->getData(i));
+}
+
 uint64_t InputSection::getTombstoneForSection(StringRef name) {
   // When a function is not live we need to update relocations referring to it.
   // If they occur in DWARF debug symbols, we want to change the pc of the
