@@ -227,8 +227,6 @@ class GdbRemoteTestCaseBase(Base):
             self.debug_monitor_exe = re.sub(r' \(deleted\)$', '', exe)
         else:
             self.debug_monitor_exe = get_lldb_server_exe()
-            if not self.debug_monitor_exe:
-                self.skipTest("lldb-server exe not found")
 
         self.debug_monitor_extra_args = ["gdbserver"]
         self.setUpServerLogging(is_llgs=True)
@@ -237,8 +235,6 @@ class GdbRemoteTestCaseBase(Base):
 
     def _init_debugserver_test(self):
         self.debug_monitor_exe = get_debugserver_exe()
-        if not self.debug_monitor_exe:
-            self.skipTest("debugserver exe not found")
         self.setUpServerLogging(is_llgs=False)
         self.reverse_connect = True
 
@@ -525,8 +521,9 @@ class GdbRemoteTestCaseBase(Base):
         server = self.connect_to_debug_monitor(attach_pid=attach_pid)
         self.assertIsNotNone(server)
 
+        self.do_handshake()
+
         # Build the expected protocol stream
-        self.add_no_ack_remote_stream()
         if inferior_env:
             for name, value in inferior_env.items():
                 self.add_set_environment_packets(name, value)
@@ -535,60 +532,13 @@ class GdbRemoteTestCaseBase(Base):
 
         return {"inferior": inferior, "server": server}
 
-    def expect_socket_recv(
-            self,
-            sock,
-            expected_content_regex
-            ):
-        response = ""
-        timeout_time = time.time() + self.DEFAULT_TIMEOUT
-
-        while not expected_content_regex.match(
-                response) and time.time() < timeout_time:
-            can_read, _, _ = select.select([sock], [], [], self.DEFAULT_TIMEOUT)
-            if can_read and sock in can_read:
-                recv_bytes = sock.recv(4096)
-                if recv_bytes:
-                    response += seven.bitcast_to_string(recv_bytes)
-
-        self.assertTrue(expected_content_regex.match(response))
-
-    def expect_socket_send(self, sock, content):
-        request_bytes_remaining = content
-        timeout_time = time.time() + self.DEFAULT_TIMEOUT
-
-        while len(request_bytes_remaining) > 0 and time.time() < timeout_time:
-            _, can_write, _ = select.select([], [sock], [], self.DEFAULT_TIMEOUT)
-            if can_write and sock in can_write:
-                written_byte_count = sock.send(request_bytes_remaining.encode())
-                request_bytes_remaining = request_bytes_remaining[
-                    written_byte_count:]
-        self.assertEqual(len(request_bytes_remaining), 0)
-
-    def do_handshake(self, stub_socket):
-        # Write the ack.
-        self.expect_socket_send(stub_socket, "+")
-
-        # Send the start no ack mode packet.
-        NO_ACK_MODE_REQUEST = "$QStartNoAckMode#b0"
-        bytes_sent = stub_socket.send(NO_ACK_MODE_REQUEST.encode())
-        self.assertEqual(bytes_sent, len(NO_ACK_MODE_REQUEST))
-
-        # Receive the ack and "OK"
-        self.expect_socket_recv(stub_socket, re.compile(
-            r"^\+\$OK#[0-9a-fA-F]{2}$"))
-
-        # Send the final ack.
-        self.expect_socket_send(stub_socket, "+")
-
-    def add_no_ack_remote_stream(self):
-        self.test_sequence.add_log_lines(
-            ["read packet: +",
-             "read packet: $QStartNoAckMode#b0",
-             "send packet: +",
-             "send packet: $OK#9a",
-             "read packet: +"],
-            True)
+    def do_handshake(self):
+        server = self._server
+        server.send_ack()
+        server.send_packet(b"QStartNoAckMode")
+        self.assertEqual(server.get_normal_packet(), b"+")
+        self.assertEqual(server.get_normal_packet(), b"OK")
+        server.send_ack()
 
     def add_verified_launch_packets(self, launch_args):
         self.test_sequence.add_log_lines(
@@ -773,7 +723,8 @@ class GdbRemoteTestCaseBase(Base):
                  "permissions",
                  "flags",
                  "name",
-                 "error"])
+                 "error",
+                 "dirty-pages"])
             self.assertIsNotNone(val)
 
         mem_region_dict["name"] = seven.unhexlify(mem_region_dict.get("name", ""))
@@ -885,9 +836,10 @@ class GdbRemoteTestCaseBase(Base):
                 "send packet: $OK#00",
             ], True)
 
-    def add_qSupported_packets(self):
+    def add_qSupported_packets(self, client_features=[]):
+        features = ''.join(';' + x for x in client_features)
         self.test_sequence.add_log_lines(
-            ["read packet: $qSupported#00",
+            ["read packet: $qSupported{}#00".format(features),
              {"direction": "send", "regex": r"^\$(.*)#[0-9a-fA-F]{2}", "capture": {1: "qSupported_response"}},
              ], True)
 
@@ -902,7 +854,11 @@ class GdbRemoteTestCaseBase(Base):
         "qXfer:libraries-svr4:read",
         "qXfer:features:read",
         "qEcho",
-        "QPassSignals"
+        "QPassSignals",
+        "multiprocess",
+        "fork-events",
+        "vfork-events",
+        "memory-tagging",
     ]
 
     def parse_qSupported_response(self, context):
