@@ -10,8 +10,7 @@ class TestCase(TestBase):
     @skipIf # rdar://problem/53931074
     def test(self):
         self.build()
-        exe = self.getBuildArtifact("a.out")
-        target = self.dbg.CreateTarget(exe)
+        target = self.createTestTarget()
         callee_break = target.BreakpointCreateByName(
             "SomeClass::SomeClass(ParamClass)", None)
         self.assertTrue(callee_break.GetNumLocations() > 0)
