@@ -23,7 +23,8 @@ class STLTestCase(TestBase):
         self.build()
 
         lldbutil.run_to_source_breakpoint(self,
-            "// Set break point at this line.", lldb.SBFileSpec("main.cpp"))
+                                          "// Set break point at this line.",
+                                          lldb.SBFileSpec("main.cpp"))
 
         # Activate importing of std module.
         self.runCmd("settings set target.import-std-module true")
