@@ -28,7 +28,7 @@ class TestStatsAPI(TestBase):
         res = stats.GetAsJSON(stream)
         stats_json = sorted(json.loads(stream.GetData()))
         self.assertEqual(len(stats_json), 4)
-        self.assertTrue("Number of expr evaluation failures" in stats_json)
-        self.assertTrue("Number of expr evaluation successes" in stats_json)
-        self.assertTrue("Number of frame var failures" in stats_json)
-        self.assertTrue("Number of frame var successes" in stats_json)
+        self.assertIn("Number of expr evaluation failures", stats_json)
+        self.assertIn("Number of expr evaluation successes", stats_json)
+        self.assertIn("Number of frame var failures", stats_json)
+        self.assertIn("Number of frame var successes", stats_json)
