@@ -14,6 +14,8 @@ class ConcurrentManyBreakpoints(ConcurrentEventsBase):
     # Atomic sequences are not supported yet for MIPS in LLDB.
     @skipIf(triple='^mips')
     @skipIfOutOfTreeDebugserver
+    @expectedFailureAll(archs=["aarch64"], oslist=["freebsd"],
+                        bugnumber="llvm.org/pr49433")
     def test(self):
         """Test 100 breakpoints from 100 threads."""
         self.build(dictionary=self.getBuildFlags())
