@@ -13,6 +13,8 @@ class ConcurrentTwoBreakpointThreads(ConcurrentEventsBase):
 
     # Atomic sequences are not supported yet for MIPS in LLDB.
     @skipIf(triple='^mips')
+    @expectedFailureAll(archs=["aarch64"], oslist=["freebsd"],
+                        bugnumber="llvm.org/pr49433")
     def test(self):
         """Test two threads that trigger a breakpoint. """
         self.build(dictionary=self.getBuildFlags())
