@@ -46,7 +46,7 @@ class ObjectDescriptionAPITestCase(TestBase):
         # Make sure we hit our breakpoint:
         thread_list = lldbutil.get_threads_stopped_at_breakpoint(
             process, breakpoint)
-        self.assertTrue(len(thread_list) == 1)
+        self.assertEqual(len(thread_list), 1)
 
         thread = thread_list[0]
         frame0 = thread.GetFrameAtIndex(0)
@@ -59,7 +59,7 @@ class ObjectDescriptionAPITestCase(TestBase):
                 print("val:", v)
                 print("object description:", v.GetObjectDescription())
             if v.GetName() == 'my_global_str':
-                self.assertTrue(v.GetObjectDescription() ==
+                self.assertEqual(v.GetObjectDescription(),
                                 'This is a global string')
 
         # But not here!
@@ -70,5 +70,5 @@ class ObjectDescriptionAPITestCase(TestBase):
                 print("val:", v)
                 print("object description:", v.GetObjectDescription())
             if v.GetName() == 'my_global_str':
-                self.assertTrue(v.GetObjectDescription() ==
+                self.assertEqual(v.GetObjectDescription(),
                                 'This is a global string')
