@@ -18,6 +18,7 @@
 #include "llvm/ADT/StringExtras.h"
 #include "llvm/ADT/StringRef.h"
 #include "llvm/ADT/Triple.h"
+#include "llvm/Config/config.h"
 #include "llvm/IR/Constant.h"
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Function.h"
@@ -76,43 +77,48 @@ static cl::opt<unsigned> StaticFuncStripDirNamePrefix(
 static std::string getInstrProfErrString(instrprof_error Err) {
   switch (Err) {
   case instrprof_error::success:
-    return "Success";
+    return "success";
   case instrprof_error::eof:
-    return "End of File";
+    return "end of File";
   case instrprof_error::unrecognized_format:
-    return "Unrecognized instrumentation profile encoding format";
+    return "unrecognized instrumentation profile encoding format";
   case instrprof_error::bad_magic:
-    return "Invalid instrumentation profile data (bad magic)";
+    return "invalid instrumentation profile data (bad magic)";
   case instrprof_error::bad_header:
-    return "Invalid instrumentation profile data (file header is corrupt)";
+    return "invalid instrumentation profile data (file header is corrupt)";
   case instrprof_error::unsupported_version:
-    return "Unsupported instrumentation profile format version";
+    return "unsupported instrumentation profile format version";
   case instrprof_error::unsupported_hash_type:
-    return "Unsupported instrumentation profile hash type";
+    return "unsupported instrumentation profile hash type";
   case instrprof_error::too_large:
-    return "Too much profile data";
+    return "too much profile data";
   case instrprof_error::truncated:
-    return "Truncated profile data";
+    return "truncated profile data";
   case instrprof_error::malformed:
-    return "Malformed instrumentation profile data";
+    return "malformed instrumentation profile data";
+  case instrprof_error::invalid_prof:
+    return "invalid profile created. Please file a bug "
+           "at: " BUG_REPORT_URL
+           " and include the profraw files that caused this error.";
   case instrprof_error::unknown_function:
-    return "No profile data available for function";
+    return "no profile data available for function";
   case instrprof_error::hash_mismatch:
-    return "Function control flow change detected (hash mismatch)";
+    return "function control flow change detected (hash mismatch)";
   case instrprof_error::count_mismatch:
-    return "Function basic block count change detected (counter mismatch)";
+    return "function basic block count change detected (counter mismatch)";
   case instrprof_error::counter_overflow:
-    return "Counter overflow";
+    return "counter overflow";
   case instrprof_error::value_site_count_mismatch:
-    return "Function value site count change detected (counter mismatch)";
+    return "function value site count change detected (counter mismatch)";
   case instrprof_error::compress_failed:
-    return "Failed to compress data (zlib)";
+    return "failed to compress data (zlib)";
   case instrprof_error::uncompress_failed:
-    return "Failed to uncompress data (zlib)";
+    return "failed to uncompress data (zlib)";
   case instrprof_error::empty_raw_profile:
-    return "Empty raw profile file";
+    return "empty raw profile file";
   case instrprof_error::zlib_unavailable:
-    return "Profile uses zlib compression but the profile reader was built without zlib support";
+    return "profile uses zlib compression but the profile reader was built "
+           "without zlib support";
   }
   llvm_unreachable("A value of instrprof_error has no message.");
 }
@@ -348,16 +354,29 @@ Error InstrProfSymtab::create(Module &M, bool InLTO) {
       return E;
     MD5FuncMap.emplace_back(Function::getGUID(PGOFuncName), &F);
     // In ThinLTO, local function may have been promoted to global and have
-    // suffix added to the function name. We need to add the stripped function
-    // name to the symbol table so that we can find a match from profile.
-    if (InLTO) {
-      auto pos = PGOFuncName.find('.');
-      if (pos != std::string::npos) {
-        const std::string &OtherFuncName = PGOFuncName.substr(0, pos);
-        if (Error E = addFuncName(OtherFuncName))
-          return E;
-        MD5FuncMap.emplace_back(Function::getGUID(OtherFuncName), &F);
-      }
+    // suffix ".llvm." added to the function name. We need to add the
+    // stripped function name to the symbol table so that we can find a match
+    // from profile.
+    //
+    // We may have other suffixes similar as ".llvm." which are needed to
+    // be stripped before the matching, but ".__uniq." suffix which is used
+    // to differentiate internal linkage functions in different modules
+    // should be kept. Now this is the only suffix with the pattern ".xxx"
+    // which is kept before matching.
+    const std::string UniqSuffix = ".__uniq.";
+    auto pos = PGOFuncName.find(UniqSuffix);
+    // Search '.' after ".__uniq." if ".__uniq." exists, otherwise
+    // search '.' from the beginning.
+    if (pos != std::string::npos)
+      pos += UniqSuffix.length();
+    else
+      pos = 0;
+    pos = PGOFuncName.find('.', pos);
+    if (pos != std::string::npos && pos != 0) {
+      const std::string &OtherFuncName = PGOFuncName.substr(0, pos);
+      if (Error E = addFuncName(OtherFuncName))
+        return E;
+      MD5FuncMap.emplace_back(Function::getGUID(OtherFuncName), &F);
     }
   }
   Sorted = false;
@@ -982,7 +1001,8 @@ bool getValueProfDataFromInst(const Instruction &Inst,
                               InstrProfValueKind ValueKind,
                               uint32_t MaxNumValueData,
                               InstrProfValueData ValueData[],
-                              uint32_t &ActualNumValueData, uint64_t &TotalC) {
+                              uint32_t &ActualNumValueData, uint64_t &TotalC,
+                              bool GetNoICPValue) {
   MDNode *MD = Inst.getMetadata(LLVMContext::MD_prof);
   if (!MD)
     return false;
@@ -1023,8 +1043,11 @@ bool getValueProfDataFromInst(const Instruction &Inst,
         mdconst::dyn_extract<ConstantInt>(MD->getOperand(I + 1));
     if (!Value || !Count)
       return false;
+    uint64_t CntValue = Count->getZExtValue();
+    if (!GetNoICPValue && (CntValue == NOMORE_ICP_MAGICNUM))
+      continue;
     ValueData[ActualNumValueData].Value = Value->getZExtValue();
-    ValueData[ActualNumValueData].Count = Count->getZExtValue();
+    ValueData[ActualNumValueData].Count = CntValue;
     ActualNumValueData++;
   }
   return true;
