@@ -10,6 +10,7 @@
 #define LLVM_TOOLS_OBJCOPY_OBJCOPY_H
 
 #include "llvm/Support/Error.h"
+#include "llvm/Support/raw_ostream.h"
 
 namespace llvm {
 
@@ -22,9 +23,10 @@ class Archive;
 } // end namespace object
 
 namespace objcopy {
-struct CopyConfig;
+class MultiFormatConfig;
 Expected<std::vector<NewArchiveMember>>
-createNewArchiveMembers(CopyConfig &Config, const object::Archive &Ar);
+createNewArchiveMembers(const MultiFormatConfig &Config,
+                        const object::Archive &Ar);
 
 } // end namespace objcopy
 } // end namespace llvm
