@@ -59,20 +59,20 @@ TEST(VectorTypesTest, FixedLength) {
       dyn_cast<FixedVectorType>(VectorType::get(Int32Ty, V8Int32Ty));
   EXPECT_VTY_EQ(V8Int32Ty, V8Int32Ty2);
 
-  auto *V8Int16Ty =
-      dyn_cast<FixedVectorType>(VectorType::get(Int16Ty, {8, false}));
+  auto *V8Int16Ty = dyn_cast<FixedVectorType>(
+      VectorType::get(Int16Ty, ElementCount::getFixed(8)));
   ASSERT_NE(nullptr, V8Int16Ty);
   EXPECT_EQ(V8Int16Ty->getNumElements(), 8U);
   EXPECT_EQ(V8Int16Ty->getElementType()->getScalarSizeInBits(), 16U);
 
-  ElementCount EltCnt(4, false);
+  auto EltCnt = ElementCount::getFixed(4);
   auto *V4Int64Ty = dyn_cast<FixedVectorType>(VectorType::get(Int64Ty, EltCnt));
   ASSERT_NE(nullptr, V4Int64Ty);
   EXPECT_EQ(V4Int64Ty->getNumElements(), 4U);
   EXPECT_EQ(V4Int64Ty->getElementType()->getScalarSizeInBits(), 64U);
 
-  auto *V2Int64Ty =
-      dyn_cast<FixedVectorType>(VectorType::get(Int64Ty, EltCnt / 2));
+  auto *V2Int64Ty = dyn_cast<FixedVectorType>(
+      VectorType::get(Int64Ty, EltCnt.divideCoefficientBy(2)));
   ASSERT_NE(nullptr, V2Int64Ty);
   EXPECT_EQ(V2Int64Ty->getNumElements(), 2U);
   EXPECT_EQ(V2Int64Ty->getElementType()->getScalarSizeInBits(), 64U);
@@ -119,8 +119,8 @@ TEST(VectorTypesTest, FixedLength) {
   EXPECT_EQ(ConvTy->getElementType()->getScalarSizeInBits(), 64U);
 
   EltCnt = V8Int64Ty->getElementCount();
-  EXPECT_EQ(EltCnt.Min, 8U);
-  ASSERT_FALSE(EltCnt.Scalable);
+  EXPECT_EQ(EltCnt.getKnownMinValue(), 8U);
+  ASSERT_FALSE(EltCnt.isScalable());
 }
 
 TEST(VectorTypesTest, Scalable) {
@@ -153,21 +153,21 @@ TEST(VectorTypesTest, Scalable) {
       dyn_cast<ScalableVectorType>(VectorType::get(Int32Ty, ScV8Int32Ty));
   EXPECT_VTY_EQ(ScV8Int32Ty, ScV8Int32Ty2);
 
-  auto *ScV8Int16Ty =
-      dyn_cast<ScalableVectorType>(VectorType::get(Int16Ty, {8, true}));
+  auto *ScV8Int16Ty = dyn_cast<ScalableVectorType>(
+      VectorType::get(Int16Ty, ElementCount::getScalable(8)));
   ASSERT_NE(nullptr, ScV8Int16Ty);
   EXPECT_EQ(ScV8Int16Ty->getMinNumElements(), 8U);
   EXPECT_EQ(ScV8Int16Ty->getElementType()->getScalarSizeInBits(), 16U);
 
-  ElementCount EltCnt(4, true);
+  auto EltCnt = ElementCount::getScalable(4);
   auto *ScV4Int64Ty =
       dyn_cast<ScalableVectorType>(VectorType::get(Int64Ty, EltCnt));
   ASSERT_NE(nullptr, ScV4Int64Ty);
   EXPECT_EQ(ScV4Int64Ty->getMinNumElements(), 4U);
   EXPECT_EQ(ScV4Int64Ty->getElementType()->getScalarSizeInBits(), 64U);
 
-  auto *ScV2Int64Ty =
-      dyn_cast<ScalableVectorType>(VectorType::get(Int64Ty, EltCnt / 2));
+  auto *ScV2Int64Ty = dyn_cast<ScalableVectorType>(
+      VectorType::get(Int64Ty, EltCnt.divideCoefficientBy(2)));
   ASSERT_NE(nullptr, ScV2Int64Ty);
   EXPECT_EQ(ScV2Int64Ty->getMinNumElements(), 2U);
   EXPECT_EQ(ScV2Int64Ty->getElementType()->getScalarSizeInBits(), 64U);
@@ -215,8 +215,8 @@ TEST(VectorTypesTest, Scalable) {
   EXPECT_EQ(ConvTy->getElementType()->getScalarSizeInBits(), 64U);
 
   EltCnt = ScV8Int64Ty->getElementCount();
-  EXPECT_EQ(EltCnt.Min, 8U);
-  ASSERT_TRUE(EltCnt.Scalable);
+  EXPECT_EQ(EltCnt.getKnownMinValue(), 8U);
+  ASSERT_TRUE(EltCnt.isScalable());
 }
 
 TEST(VectorTypesTest, BaseVectorType) {
@@ -225,14 +225,15 @@ TEST(VectorTypesTest, BaseVectorType) {
   Type *Int16Ty = Type::getInt16Ty(Ctx);
   Type *Int32Ty = Type::getInt32Ty(Ctx);
 
-  std::array<VectorType *, 8> VTys = {VectorType::get(Int16Ty, {4, true}),
-                                      VectorType::get(Int16Ty, {4, false}),
-                                      VectorType::get(Int16Ty, {2, true}),
-                                      VectorType::get(Int16Ty, {2, false}),
-                                      VectorType::get(Int32Ty, {4, true}),
-                                      VectorType::get(Int32Ty, {4, false}),
-                                      VectorType::get(Int32Ty, {2, true}),
-                                      VectorType::get(Int32Ty, {2, false})};
+  std::array<VectorType *, 8> VTys = {
+      VectorType::get(Int16Ty, ElementCount::getScalable(4)),
+      VectorType::get(Int16Ty, ElementCount::getFixed(4)),
+      VectorType::get(Int16Ty, ElementCount::getScalable(2)),
+      VectorType::get(Int16Ty, ElementCount::getFixed(2)),
+      VectorType::get(Int32Ty, ElementCount::getScalable(4)),
+      VectorType::get(Int32Ty, ElementCount::getFixed(4)),
+      VectorType::get(Int32Ty, ElementCount::getScalable(2)),
+      VectorType::get(Int32Ty, ElementCount::getFixed(2))};
 
   /*
     The comparison matrix is symmetric, so we only check the upper triangle:
@@ -249,7 +250,7 @@ TEST(VectorTypesTest, BaseVectorType) {
     // test I == J
     VectorType *VI = VTys[I];
     ElementCount ECI = VI->getElementCount();
-    EXPECT_EQ(isa<ScalableVectorType>(VI), ECI.Scalable);
+    EXPECT_EQ(isa<ScalableVectorType>(VI), ECI.isScalable());
 
     for (size_t J = I + 1, JEnd = VTys.size(); J < JEnd; ++J) {
       // test I < J
@@ -285,10 +286,10 @@ TEST(VectorTypesTest, FixedLenComparisons) {
   EXPECT_EQ(V2I32Len.getKnownMinSize(), 64U);
   EXPECT_FALSE(V2I32Len.isScalable());
 
-  EXPECT_LT(V2Int32Ty->getPrimitiveSizeInBits(),
-            V4Int32Ty->getPrimitiveSizeInBits());
-  EXPECT_GT(V2Int64Ty->getPrimitiveSizeInBits(),
-            V2Int32Ty->getPrimitiveSizeInBits());
+  EXPECT_LT(V2Int32Ty->getPrimitiveSizeInBits().getFixedSize(),
+            V4Int32Ty->getPrimitiveSizeInBits().getFixedSize());
+  EXPECT_GT(V2Int64Ty->getPrimitiveSizeInBits().getFixedSize(),
+            V2Int32Ty->getPrimitiveSizeInBits().getFixedSize());
   EXPECT_EQ(V4Int32Ty->getPrimitiveSizeInBits(),
             V2Int64Ty->getPrimitiveSizeInBits());
   EXPECT_NE(V2Int32Ty->getPrimitiveSizeInBits(),
@@ -331,14 +332,14 @@ TEST(VectorTypesTest, ScalableComparisons) {
   EXPECT_EQ(ScV2I32Len.getKnownMinSize(), 64U);
   EXPECT_TRUE(ScV2I32Len.isScalable());
 
-  EXPECT_LT(ScV2Int32Ty->getPrimitiveSizeInBits(),
-            ScV4Int32Ty->getPrimitiveSizeInBits());
-  EXPECT_GT(ScV2Int64Ty->getPrimitiveSizeInBits(),
-            ScV2Int32Ty->getPrimitiveSizeInBits());
-  EXPECT_EQ(ScV4Int32Ty->getPrimitiveSizeInBits(),
-            ScV2Int64Ty->getPrimitiveSizeInBits());
-  EXPECT_NE(ScV2Int32Ty->getPrimitiveSizeInBits(),
-            ScV2Int64Ty->getPrimitiveSizeInBits());
+  EXPECT_LT(ScV2Int32Ty->getPrimitiveSizeInBits().getKnownMinSize(),
+            ScV4Int32Ty->getPrimitiveSizeInBits().getKnownMinSize());
+  EXPECT_GT(ScV2Int64Ty->getPrimitiveSizeInBits().getKnownMinSize(),
+            ScV2Int32Ty->getPrimitiveSizeInBits().getKnownMinSize());
+  EXPECT_EQ(ScV4Int32Ty->getPrimitiveSizeInBits().getKnownMinSize(),
+            ScV2Int64Ty->getPrimitiveSizeInBits().getKnownMinSize());
+  EXPECT_NE(ScV2Int32Ty->getPrimitiveSizeInBits().getKnownMinSize(),
+            ScV2Int64Ty->getPrimitiveSizeInBits().getKnownMinSize());
 
   // Check the DataLayout interfaces.
   EXPECT_EQ(DL.getTypeSizeInBits(ScV2Int64Ty),
