@@ -22,9 +22,17 @@
 //===----------------------------------------------------------------------===//
 // Visibility annotations.
 // Use MLIR_CAPI_EXPORTED for exported functions.
+//
+// On Windows, if MLIR_CAPI_ENABLE_WINDOWS_DLL_DECLSPEC is defined, then
+// __declspec(dllexport) and __declspec(dllimport) will be generated. This
+// can only be enabled if actually building DLLs. It is generally, mutually
+// exclusive with the use of other mechanisms for managing imports/exports
+// (i.e. CMake's WINDOWS_EXPORT_ALL_SYMBOLS feature).
 //===----------------------------------------------------------------------===//
 
-#if defined(MLIR_CAPI_DISABLE_VISIBILITY_ANNOTATIONS)
+#if (defined(_WIN32) || defined(__CYGWIN__)) &&                                \
+    !defined(MLIR_CAPI_ENABLE_WINDOWS_DLL_DECLSPEC)
+// Visibility annotations disabled.
 #define MLIR_CAPI_EXPORTED
 #elif defined(_WIN32) || defined(__CYGWIN__)
 // Windows visibility declarations.
