@@ -15,6 +15,7 @@
 #define MLIR_DIALECT_LLVMIR_LLVMTYPES_H_
 
 #include "mlir/IR/Types.h"
+#include "mlir/Interfaces/DataLayoutInterfaces.h"
 
 namespace llvm {
 class ElementCount;
@@ -35,6 +36,13 @@ struct LLVMPointerTypeStorage;
 struct LLVMStructTypeStorage;
 struct LLVMTypeAndSizeStorage;
 } // namespace detail
+} // namespace LLVM
+} // namespace mlir
+
+#include "mlir/Dialect/LLVMIR/LLVMTypeInterfaces.h.inc"
+
+namespace mlir {
+namespace LLVM {
 
 //===----------------------------------------------------------------------===//
 // Trivial types.
@@ -68,6 +76,7 @@ class LLVMArrayType : public Type::TypeBase<LLVMArrayType, Type,
 public:
   /// Inherit base constructors.
   using Base::Base;
+  using Base::getChecked;
 
   /// Checks if the given type can be used inside an array type.
   static bool isValidElementType(Type type);
@@ -75,8 +84,8 @@ public:
   /// Gets or creates an instance of LLVM dialect array type containing
   /// `numElements` of `elementType`, in the same context as `elementType`.
   static LLVMArrayType get(Type elementType, unsigned numElements);
-  static LLVMArrayType getChecked(Location loc, Type elementType,
-                                  unsigned numElements);
+  static LLVMArrayType getChecked(function_ref<InFlightDiagnostic()> emitError,
+                                  Type elementType, unsigned numElements);
 
   /// Returns the element type of the array.
   Type getElementType();
@@ -85,9 +94,8 @@ public:
   unsigned getNumElements();
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location loc,
-                                                    Type elementType,
-                                                    unsigned numElements);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              Type elementType, unsigned numElements);
 };
 
 //===----------------------------------------------------------------------===//
@@ -103,6 +111,7 @@ class LLVMFunctionType
 public:
   /// Inherit base constructors.
   using Base::Base;
+  using Base::getChecked;
 
   /// Checks if the given type can be used an argument in a function type.
   static bool isValidArgumentType(Type type);
@@ -117,9 +126,9 @@ public:
   /// as the `result` type.
   static LLVMFunctionType get(Type result, ArrayRef<Type> arguments,
                               bool isVarArg = false);
-  static LLVMFunctionType getChecked(Location loc, Type result,
-                                     ArrayRef<Type> arguments,
-                                     bool isVarArg = false);
+  static LLVMFunctionType
+  getChecked(function_ref<InFlightDiagnostic()> emitError, Type result,
+             ArrayRef<Type> arguments, bool isVarArg = false);
 
   /// Returns the result type of the function.
   Type getReturnType();
@@ -135,9 +144,8 @@ public:
   ArrayRef<Type> params() { return getParams(); }
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location loc, Type result,
-                                                    ArrayRef<Type> arguments,
-                                                    bool);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              Type result, ArrayRef<Type> arguments, bool);
 };
 
 //===----------------------------------------------------------------------===//
@@ -148,10 +156,12 @@ public:
 /// object in memory. It is parameterized by the element type and the address
 /// space.
 class LLVMPointerType : public Type::TypeBase<LLVMPointerType, Type,
-                                              detail::LLVMPointerTypeStorage> {
+                                              detail::LLVMPointerTypeStorage,
+                                              DataLayoutTypeInterface::Trait> {
 public:
   /// Inherit base constructors.
   using Base::Base;
+  using Base::getChecked;
 
   /// Checks if the given type can have a pointer type pointing to it.
   static bool isValidElementType(Type type);
@@ -160,18 +170,32 @@ public:
   /// object of `pointee` type in the given address space. The pointer type is
   /// created in the same context as `pointee`.
   static LLVMPointerType get(Type pointee, unsigned addressSpace = 0);
-  static LLVMPointerType getChecked(Location loc, Type pointee,
-                                    unsigned addressSpace = 0);
+  static LLVMPointerType
+  getChecked(function_ref<InFlightDiagnostic()> emitError, Type pointee,
+             unsigned addressSpace = 0);
 
   /// Returns the pointed-to type.
-  Type getElementType();
+  Type getElementType() const;
 
   /// Returns the address space of the pointer.
-  unsigned getAddressSpace();
+  unsigned getAddressSpace() const;
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location loc, Type pointee,
-                                                    unsigned);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              Type pointee, unsigned);
+
+  /// Hooks for DataLayoutTypeInterface. Should not be called directly. Obtain a
+  /// DataLayout instance and query it instead.
+  unsigned getTypeSizeInBits(const DataLayout &dataLayout,
+                             DataLayoutEntryListRef params) const;
+  unsigned getABIAlignment(const DataLayout &dataLayout,
+                           DataLayoutEntryListRef params) const;
+  unsigned getPreferredAlignment(const DataLayout &dataLayout,
+                                 DataLayoutEntryListRef params) const;
+  bool areCompatible(DataLayoutEntryListRef oldLayout,
+                     DataLayoutEntryListRef newLayout) const;
+  LogicalResult verifyEntries(DataLayoutEntryListRef entries,
+                              Location loc) const;
 };
 
 //===----------------------------------------------------------------------===//
@@ -217,7 +241,9 @@ public:
   /// in the context. Instead, it will just return the existing struct,
   /// similarly to the rest of MLIR type ::get methods.
   static LLVMStructType getIdentified(MLIRContext *context, StringRef name);
-  static LLVMStructType getIdentifiedChecked(Location loc, StringRef name);
+  static LLVMStructType
+  getIdentifiedChecked(function_ref<InFlightDiagnostic()> emitError,
+                       MLIRContext *context, StringRef name);
 
   /// Gets a new identified struct with the given body. The body _cannot_ be
   /// changed later. If a struct with the given name already exists, renames
@@ -231,8 +257,10 @@ public:
   /// context.
   static LLVMStructType getLiteral(MLIRContext *context, ArrayRef<Type> types,
                                    bool isPacked = false);
-  static LLVMStructType getLiteralChecked(Location loc, ArrayRef<Type> types,
-                                          bool isPacked = false);
+  static LLVMStructType
+  getLiteralChecked(function_ref<InFlightDiagnostic()> emitError,
+                    MLIRContext *context, ArrayRef<Type> types,
+                    bool isPacked = false);
 
   /// Gets or creates an intentionally-opaque identified struct. Such a struct
   /// cannot have its body set. To create an opaque struct with a mutable body,
@@ -241,7 +269,9 @@ public:
   /// already exists in the context. Instead, it will just return the existing
   /// struct, similarly to the rest of MLIR type ::get methods.
   static LLVMStructType getOpaque(StringRef name, MLIRContext *context);
-  static LLVMStructType getOpaqueChecked(Location loc, StringRef name);
+  static LLVMStructType
+  getOpaqueChecked(function_ref<InFlightDiagnostic()> emitError,
+                   MLIRContext *context, StringRef name);
 
   /// Set the body of an identified struct. Returns failure if the body could
   /// not be set, e.g. if the struct already has a body or if it was marked as
@@ -270,9 +300,10 @@ public:
   ArrayRef<Type> getBody();
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location, StringRef, bool);
-  static LogicalResult verifyConstructionInvariants(Location loc,
-                                                    ArrayRef<Type> types, bool);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              StringRef, bool);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              ArrayRef<Type> types, bool);
 };
 
 //===----------------------------------------------------------------------===//
@@ -300,9 +331,8 @@ public:
   llvm::ElementCount getElementCount();
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location loc,
-                                                    Type elementType,
-                                                    unsigned numElements);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              Type elementType, unsigned numElements);
 };
 
 //===----------------------------------------------------------------------===//
@@ -317,12 +347,14 @@ class LLVMFixedVectorType
 public:
   /// Inherit base constructor.
   using Base::Base;
+  using Base::getChecked;
 
   /// Gets or creates a fixed vector type containing `numElements` of
   /// `elementType` in the same context as `elementType`.
   static LLVMFixedVectorType get(Type elementType, unsigned numElements);
-  static LLVMFixedVectorType getChecked(Location loc, Type elementType,
-                                        unsigned numElements);
+  static LLVMFixedVectorType
+  getChecked(function_ref<InFlightDiagnostic()> emitError, Type elementType,
+             unsigned numElements);
 
   /// Checks if the given type can be used in a vector type. This type supports
   /// only a subset of LLVM dialect types that don't have a built-in
@@ -336,9 +368,8 @@ public:
   unsigned getNumElements();
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location loc,
-                                                    Type elementType,
-                                                    unsigned numElements);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              Type elementType, unsigned numElements);
 };
 
 //===----------------------------------------------------------------------===//
@@ -354,12 +385,14 @@ class LLVMScalableVectorType
 public:
   /// Inherit base constructor.
   using Base::Base;
+  using Base::getChecked;
 
   /// Gets or creates a scalable vector type containing a non-zero multiple of
   /// `minNumElements` of `elementType` in the same context as `elementType`.
   static LLVMScalableVectorType get(Type elementType, unsigned minNumElements);
-  static LLVMScalableVectorType getChecked(Location loc, Type elementType,
-                                           unsigned minNumElements);
+  static LLVMScalableVectorType
+  getChecked(function_ref<InFlightDiagnostic()> emitError, Type elementType,
+             unsigned minNumElements);
 
   /// Checks if the given type can be used in a vector type.
   static bool isValidElementType(Type type);
@@ -373,9 +406,8 @@ public:
   unsigned getMinNumElements();
 
   /// Verifies that the type about to be constructed is well-formed.
-  static LogicalResult verifyConstructionInvariants(Location loc,
-                                                    Type elementType,
-                                                    unsigned minNumElements);
+  static LogicalResult verify(function_ref<InFlightDiagnostic()> emitError,
+                              Type elementType, unsigned minNumElements);
 };
 
 //===----------------------------------------------------------------------===//
