@@ -20,11 +20,6 @@ void registerFromLLVMIRTranslation();
 void registerFromSPIRVTranslation();
 void registerToLLVMIRTranslation();
 void registerToSPIRVTranslation();
-void registerToNVVMIRTranslation();
-void registerToROCDLIRTranslation();
-void registerArmNeonToLLVMIRTranslation();
-void registerAVX512ToLLVMIRTranslation();
-void registerArmSVEToLLVMIRTranslation();
 
 // This function should be called before creating any MLIRContext if one
 // expects all the possible translations to be made available to the context
@@ -35,11 +30,6 @@ inline void registerAllTranslations() {
     registerFromSPIRVTranslation();
     registerToLLVMIRTranslation();
     registerToSPIRVTranslation();
-    registerToNVVMIRTranslation();
-    registerToROCDLIRTranslation();
-    registerArmNeonToLLVMIRTranslation();
-    registerAVX512ToLLVMIRTranslation();
-    registerArmSVEToLLVMIRTranslation();
     return true;
   }();
   (void)initOnce;
