@@ -13,21 +13,21 @@
 #ifndef MLIR_TARGET_SPIRV_DESERIALIZATION_H
 #define MLIR_TARGET_SPIRV_DESERIALIZATION_H
 
+#include "mlir/IR/OwningOpRef.h"
 #include "mlir/Support/LLVM.h"
 
 namespace mlir {
-struct LogicalResult;
 class MLIRContext;
 
 namespace spirv {
-class OwningSPIRVModuleRef;
+class ModuleOp;
 
 /// Deserializes the given SPIR-V `binary` module and creates a MLIR ModuleOp
 /// in the given `context`. Returns the ModuleOp on success; otherwise, reports
 /// errors to the error handler registered with `context` and returns a null
 /// module.
-OwningSPIRVModuleRef deserialize(ArrayRef<uint32_t> binary,
-                                 MLIRContext *context);
+OwningOpRef<spirv::ModuleOp> deserialize(ArrayRef<uint32_t> binary,
+                                         MLIRContext *context);
 
 } // end namespace spirv
 } // end namespace mlir
