@@ -12,6 +12,7 @@
 #include "mlir/Dialect/PDLInterp/IR/PDLInterp.h"
 #include "mlir/IR/BuiltinOps.h"
 #include "mlir/Interfaces/InferTypeOpInterface.h"
+#include "llvm/ADT/TypeSwitch.h"
 
 using namespace mlir;
 using namespace mlir::pdl_to_pdl_interp;
@@ -20,9 +21,183 @@ using namespace mlir::pdl_to_pdl_interp;
 // Predicate List Building
 //===----------------------------------------------------------------------===//
 
+static void getTreePredicates(std::vector<PositionalPredicate> &predList,
+                              Value val, PredicateBuilder &builder,
+                              DenseMap<Value, Position *> &inputs,
+                              Position *pos);
+
 /// Compares the depths of two positions.
 static bool comparePosDepth(Position *lhs, Position *rhs) {
-  return lhs->getIndex().size() < rhs->getIndex().size();
+  return lhs->getOperationDepth() < rhs->getOperationDepth();
+}
+
+/// Returns the number of non-range elements within `values`.
+static unsigned getNumNonRangeValues(ValueRange values) {
+  return llvm::count_if(values.getTypes(),
+                        [](Type type) { return !type.isa<pdl::RangeType>(); });
+}
+
+static void getTreePredicates(std::vector<PositionalPredicate> &predList,
+                              Value val, PredicateBuilder &builder,
+                              DenseMap<Value, Position *> &inputs,
+                              AttributePosition *pos) {
+  assert(val.getType().isa<pdl::AttributeType>() && "expected attribute type");
+  pdl::AttributeOp attr = cast<pdl::AttributeOp>(val.getDefiningOp());
+  predList.emplace_back(pos, builder.getIsNotNull());
+
+  // If the attribute has a type or value, add a constraint.
+  if (Value type = attr.type())
+    getTreePredicates(predList, type, builder, inputs, builder.getType(pos));
+  else if (Attribute value = attr.valueAttr())
+    predList.emplace_back(pos, builder.getAttributeConstraint(value));
+}
+
+/// Collect all of the predicates for the given operand position.
+static void getOperandTreePredicates(std::vector<PositionalPredicate> &predList,
+                                     Value val, PredicateBuilder &builder,
+                                     DenseMap<Value, Position *> &inputs,
+                                     Position *pos) {
+  Type valueType = val.getType();
+  bool isVariadic = valueType.isa<pdl::RangeType>();
+
+  // If this is a typed operand, add a type constraint.
+  TypeSwitch<Operation *>(val.getDefiningOp())
+      .Case<pdl::OperandOp, pdl::OperandsOp>([&](auto op) {
+        // Prevent traversal into a null value if the operand has a proper
+        // index.
+        if (std::is_same<pdl::OperandOp, decltype(op)>::value ||
+            cast<OperandGroupPosition>(pos)->getOperandGroupNumber())
+          predList.emplace_back(pos, builder.getIsNotNull());
+
+        if (Value type = op.type())
+          getTreePredicates(predList, type, builder, inputs,
+                            builder.getType(pos));
+      })
+      .Case<pdl::ResultOp, pdl::ResultsOp>([&](auto op) {
+        Optional<unsigned> index = op.index();
+
+        // Prevent traversal into a null value if the result has a proper index.
+        if (index)
+          predList.emplace_back(pos, builder.getIsNotNull());
+
+        // Get the parent operation of this operand.
+        OperationPosition *parentPos = builder.getOperandDefiningOp(pos);
+        predList.emplace_back(parentPos, builder.getIsNotNull());
+
+        // Ensure that the operands match the corresponding results of the
+        // parent operation.
+        Position *resultPos = nullptr;
+        if (std::is_same<pdl::ResultOp, decltype(op)>::value)
+          resultPos = builder.getResult(parentPos, *index);
+        else
+          resultPos = builder.getResultGroup(parentPos, index, isVariadic);
+        predList.emplace_back(resultPos, builder.getEqualTo(pos));
+
+        // Collect the predicates of the parent operation.
+        getTreePredicates(predList, op.parent(), builder, inputs,
+                          (Position *)parentPos);
+      });
+}
+
+static void getTreePredicates(std::vector<PositionalPredicate> &predList,
+                              Value val, PredicateBuilder &builder,
+                              DenseMap<Value, Position *> &inputs,
+                              OperationPosition *pos) {
+  assert(val.getType().isa<pdl::OperationType>() && "expected operation");
+  pdl::OperationOp op = cast<pdl::OperationOp>(val.getDefiningOp());
+  OperationPosition *opPos = cast<OperationPosition>(pos);
+
+  // Ensure getDefiningOp returns a non-null operation.
+  if (!opPos->isRoot())
+    predList.emplace_back(pos, builder.getIsNotNull());
+
+  // Check that this is the correct root operation.
+  if (Optional<StringRef> opName = op.name())
+    predList.emplace_back(pos, builder.getOperationName(*opName));
+
+  // Check that the operation has the proper number of operands. If there are
+  // any variable length operands, we check a minimum instead of an exact count.
+  OperandRange operands = op.operands();
+  unsigned minOperands = getNumNonRangeValues(operands);
+  if (minOperands != operands.size()) {
+    if (minOperands)
+      predList.emplace_back(pos, builder.getOperandCountAtLeast(minOperands));
+  } else {
+    predList.emplace_back(pos, builder.getOperandCount(minOperands));
+  }
+
+  // Check that the operation has the proper number of results. If there are
+  // any variable length results, we check a minimum instead of an exact count.
+  OperandRange types = op.types();
+  unsigned minResults = getNumNonRangeValues(types);
+  if (minResults == types.size())
+    predList.emplace_back(pos, builder.getResultCount(types.size()));
+  else if (minResults)
+    predList.emplace_back(pos, builder.getResultCountAtLeast(minResults));
+
+  // Recurse into any attributes, operands, or results.
+  for (auto it : llvm::zip(op.attributeNames(), op.attributes())) {
+    getTreePredicates(
+        predList, std::get<1>(it), builder, inputs,
+        builder.getAttribute(opPos,
+                             std::get<0>(it).cast<StringAttr>().getValue()));
+  }
+
+  // Process the operands and results of the operation. For all values up to
+  // the first variable length value, we use the concrete operand/result
+  // number. After that, we use the "group" given that we can't know the
+  // concrete indices until runtime. If there is only one variadic operand
+  // group, we treat it as all of the operands/results of the operation.
+  /// Operands.
+  if (operands.size() == 1 && operands[0].getType().isa<pdl::RangeType>()) {
+    getTreePredicates(predList, operands.front(), builder, inputs,
+                      builder.getAllOperands(opPos));
+  } else {
+    bool foundVariableLength = false;
+    for (auto operandIt : llvm::enumerate(operands)) {
+      bool isVariadic = operandIt.value().getType().isa<pdl::RangeType>();
+      foundVariableLength |= isVariadic;
+
+      Position *pos =
+          foundVariableLength
+              ? builder.getOperandGroup(opPos, operandIt.index(), isVariadic)
+              : builder.getOperand(opPos, operandIt.index());
+      getTreePredicates(predList, operandIt.value(), builder, inputs, pos);
+    }
+  }
+  /// Results.
+  if (types.size() == 1 && types[0].getType().isa<pdl::RangeType>()) {
+    getTreePredicates(predList, types.front(), builder, inputs,
+                      builder.getType(builder.getAllResults(opPos)));
+  } else {
+    bool foundVariableLength = false;
+    for (auto &resultIt : llvm::enumerate(types)) {
+      bool isVariadic = resultIt.value().getType().isa<pdl::RangeType>();
+      foundVariableLength |= isVariadic;
+
+      auto *resultPos =
+          foundVariableLength
+              ? builder.getResultGroup(pos, resultIt.index(), isVariadic)
+              : builder.getResult(pos, resultIt.index());
+      predList.emplace_back(resultPos, builder.getIsNotNull());
+      getTreePredicates(predList, resultIt.value(), builder, inputs,
+                        builder.getType(resultPos));
+    }
+  }
+}
+
+static void getTreePredicates(std::vector<PositionalPredicate> &predList,
+                              Value val, PredicateBuilder &builder,
+                              DenseMap<Value, Position *> &inputs,
+                              TypePosition *pos) {
+  // Check for a constraint on a constant type.
+  if (pdl::TypeOp typeOp = val.getDefiningOp<pdl::TypeOp>()) {
+    if (Attribute type = typeOp.typeAttr())
+      predList.emplace_back(pos, builder.getTypeConstraint(type));
+  } else if (pdl::TypesOp typeOp = val.getDefiningOp<pdl::TypesOp>()) {
+    if (Attribute typeAttr = typeOp.typesAttr())
+      predList.emplace_back(pos, builder.getTypeConstraint(typeAttr));
+  }
 }
 
 /// Collect the tree predicates anchored at the given value.
@@ -32,139 +207,96 @@ static void getTreePredicates(std::vector<PositionalPredicate> &predList,
                               Position *pos) {
   // Make sure this input value is accessible to the rewrite.
   auto it = inputs.try_emplace(val, pos);
-
-  // If this is an input value that has been visited in the tree, add a
-  // constraint to ensure that both instances refer to the same value.
-  if (!it.second &&
-      isa<pdl::AttributeOp, pdl::InputOp, pdl::TypeOp>(val.getDefiningOp())) {
-    auto minMaxPositions = std::minmax(pos, it.first->second, comparePosDepth);
-    predList.emplace_back(minMaxPositions.second,
-                          builder.getEqualTo(minMaxPositions.first));
+  if (!it.second) {
+    // If this is an input value that has been visited in the tree, add a
+    // constraint to ensure that both instances refer to the same value.
+    if (isa<pdl::AttributeOp, pdl::OperandOp, pdl::OperandsOp, pdl::OperationOp,
+            pdl::TypeOp>(val.getDefiningOp())) {
+      auto minMaxPositions =
+          std::minmax(pos, it.first->second, comparePosDepth);
+      predList.emplace_back(minMaxPositions.second,
+                            builder.getEqualTo(minMaxPositions.first));
+    }
     return;
   }
 
-  // Check for a per-position predicate to apply.
-  switch (pos->getKind()) {
-  case Predicates::AttributePos: {
-    assert(val.getType().isa<pdl::AttributeType>() &&
-           "expected attribute type");
-    pdl::AttributeOp attr = cast<pdl::AttributeOp>(val.getDefiningOp());
-    predList.emplace_back(pos, builder.getIsNotNull());
-
-    // If the attribute has a type, add a type constraint.
-    if (Value type = attr.type()) {
-      getTreePredicates(predList, type, builder, inputs, builder.getType(pos));
-
-      // Check for a constant value of the attribute.
-    } else if (Optional<Attribute> value = attr.value()) {
-      predList.emplace_back(pos, builder.getAttributeConstraint(*value));
-    }
-    break;
-  }
-  case Predicates::OperandPos: {
-    assert(val.getType().isa<pdl::ValueType>() && "expected value type");
-
-    // Prevent traversal into a null value.
-    predList.emplace_back(pos, builder.getIsNotNull());
-
-    // If this is a typed input, add a type constraint.
-    if (auto in = val.getDefiningOp<pdl::InputOp>()) {
-      if (Value type = in.type()) {
-        getTreePredicates(predList, type, builder, inputs,
-                          builder.getType(pos));
-      }
-
-      // Otherwise, recurse into the parent node.
-    } else if (auto parentOp = val.getDefiningOp<pdl::OperationOp>()) {
-      getTreePredicates(predList, parentOp.op(), builder, inputs,
-                        builder.getParent(cast<OperandPosition>(pos)));
-    }
-    break;
-  }
-  case Predicates::OperationPos: {
-    assert(val.getType().isa<pdl::OperationType>() && "expected operation");
-    pdl::OperationOp op = cast<pdl::OperationOp>(val.getDefiningOp());
-    OperationPosition *opPos = cast<OperationPosition>(pos);
-
-    // Ensure getDefiningOp returns a non-null operation.
-    if (!opPos->isRoot())
-      predList.emplace_back(pos, builder.getIsNotNull());
-
-    // Check that this is the correct root operation.
-    if (Optional<StringRef> opName = op.name())
-      predList.emplace_back(pos, builder.getOperationName(*opName));
-
-    // Check that the operation has the proper number of operands and results.
-    OperandRange operands = op.operands();
-    ResultRange results = op.results();
-    predList.emplace_back(pos, builder.getOperandCount(operands.size()));
-    predList.emplace_back(pos, builder.getResultCount(results.size()));
-
-    // Recurse into any attributes, operands, or results.
-    for (auto it : llvm::zip(op.attributeNames(), op.attributes())) {
-      getTreePredicates(
-          predList, std::get<1>(it), builder, inputs,
-          builder.getAttribute(opPos,
-                               std::get<0>(it).cast<StringAttr>().getValue()));
-    }
-    for (auto operandIt : llvm::enumerate(operands))
-      getTreePredicates(predList, operandIt.value(), builder, inputs,
-                        builder.getOperand(opPos, operandIt.index()));
-
-    // Only recurse into results that are not referenced in the source tree.
-    for (auto resultIt : llvm::enumerate(results)) {
-      getTreePredicates(predList, resultIt.value(), builder, inputs,
-                        builder.getResult(opPos, resultIt.index()));
-    }
-    break;
-  }
-  case Predicates::ResultPos: {
-    assert(val.getType().isa<pdl::ValueType>() && "expected value type");
-    pdl::OperationOp parentOp = cast<pdl::OperationOp>(val.getDefiningOp());
-
-    // Prevent traversing a null value.
-    predList.emplace_back(pos, builder.getIsNotNull());
-
-    // Traverse the type constraint.
-    unsigned resultNo = cast<ResultPosition>(pos)->getResultNumber();
-    getTreePredicates(predList, parentOp.types()[resultNo], builder, inputs,
-                      builder.getType(pos));
-    break;
-  }
-  case Predicates::TypePos: {
-    assert(val.getType().isa<pdl::TypeType>() && "expected value type");
-    pdl::TypeOp typeOp = cast<pdl::TypeOp>(val.getDefiningOp());
-
-    // Check for a constraint on a constant type.
-    if (Optional<Type> type = typeOp.type())
-      predList.emplace_back(pos, builder.getTypeConstraint(*type));
-    break;
-  }
-  default:
-    llvm_unreachable("unknown position kind");
-  }
+  TypeSwitch<Position *>(pos)
+      .Case<AttributePosition, OperationPosition, TypePosition>([&](auto *pos) {
+        getTreePredicates(predList, val, builder, inputs, pos);
+      })
+      .Case<OperandPosition, OperandGroupPosition>([&](auto *pos) {
+        getOperandTreePredicates(predList, val, builder, inputs, pos);
+      })
+      .Default([](auto *) { llvm_unreachable("unexpected position kind"); });
 }
 
 /// Collect all of the predicates related to constraints within the given
 /// pattern operation.
-static void collectConstraintPredicates(
-    pdl::PatternOp pattern, std::vector<PositionalPredicate> &predList,
-    PredicateBuilder &builder, DenseMap<Value, Position *> &inputs) {
-  for (auto op : pattern.body().getOps<pdl::ApplyConstraintOp>()) {
-    OperandRange arguments = op.args();
-    ArrayAttr parameters = op.constParamsAttr();
+static void getConstraintPredicates(pdl::ApplyNativeConstraintOp op,
+                                    std::vector<PositionalPredicate> &predList,
+                                    PredicateBuilder &builder,
+                                    DenseMap<Value, Position *> &inputs) {
+  OperandRange arguments = op.args();
+  ArrayAttr parameters = op.constParamsAttr();
 
-    std::vector<Position *> allPositions;
-    allPositions.reserve(arguments.size());
-    for (Value arg : arguments)
-      allPositions.push_back(inputs.lookup(arg));
+  std::vector<Position *> allPositions;
+  allPositions.reserve(arguments.size());
+  for (Value arg : arguments)
+    allPositions.push_back(inputs.lookup(arg));
 
-    // Push the constraint to the furthest position.
-    Position *pos = *std::max_element(allPositions.begin(), allPositions.end(),
-                                      comparePosDepth);
-    PredicateBuilder::Predicate pred =
-        builder.getConstraint(op.name(), std::move(allPositions), parameters);
-    predList.emplace_back(pos, pred);
+  // Push the constraint to the furthest position.
+  Position *pos = *std::max_element(allPositions.begin(), allPositions.end(),
+                                    comparePosDepth);
+  PredicateBuilder::Predicate pred =
+      builder.getConstraint(op.name(), std::move(allPositions), parameters);
+  predList.emplace_back(pos, pred);
+}
+
+static void getResultPredicates(pdl::ResultOp op,
+                                std::vector<PositionalPredicate> &predList,
+                                PredicateBuilder &builder,
+                                DenseMap<Value, Position *> &inputs) {
+  Position *&resultPos = inputs[op];
+  if (resultPos)
+    return;
+
+  // Ensure that the result isn't null.
+  auto *parentPos = cast<OperationPosition>(inputs.lookup(op.parent()));
+  resultPos = builder.getResult(parentPos, op.index());
+  predList.emplace_back(resultPos, builder.getIsNotNull());
+}
+
+static void getResultPredicates(pdl::ResultsOp op,
+                                std::vector<PositionalPredicate> &predList,
+                                PredicateBuilder &builder,
+                                DenseMap<Value, Position *> &inputs) {
+  Position *&resultPos = inputs[op];
+  if (resultPos)
+    return;
+
+  // Ensure that the result isn't null if the result has an index.
+  auto *parentPos = cast<OperationPosition>(inputs.lookup(op.parent()));
+  bool isVariadic = op.getType().isa<pdl::RangeType>();
+  Optional<unsigned> index = op.index();
+  resultPos = builder.getResultGroup(parentPos, index, isVariadic);
+  if (index)
+    predList.emplace_back(resultPos, builder.getIsNotNull());
+}
+
+/// Collect all of the predicates that cannot be determined via walking the
+/// tree.
+static void getNonTreePredicates(pdl::PatternOp pattern,
+                                 std::vector<PositionalPredicate> &predList,
+                                 PredicateBuilder &builder,
+                                 DenseMap<Value, Position *> &inputs) {
+  for (Operation &op : pattern.body().getOps()) {
+    TypeSwitch<Operation *>(&op)
+        .Case<pdl::ApplyNativeConstraintOp>([&](auto constraintOp) {
+          getConstraintPredicates(constraintOp, predList, builder, inputs);
+        })
+        .Case<pdl::ResultOp, pdl::ResultsOp>([&](auto resultOp) {
+          getResultPredicates(resultOp, predList, builder, inputs);
+        });
   }
 }
 
@@ -176,7 +308,7 @@ static void buildPredicateList(pdl::PatternOp pattern,
                                DenseMap<Value, Position *> &valueToPosition) {
   getTreePredicates(predList, pattern.getRewriter().root(), builder,
                     valueToPosition, builder.getRoot());
-  collectConstraintPredicates(pattern, predList, builder, valueToPosition);
+  getNonTreePredicates(pattern, predList, builder, valueToPosition);
 }
 
 //===----------------------------------------------------------------------===//
@@ -214,20 +346,20 @@ struct OrderedPredicate {
   /// within that pattern.
   DenseMap<Operation *, Qualifier *> patternToAnswer;
 
-  /// Returns true if this predicate is ordered before `other`, based on the
-  /// cost model.
-  bool operator<(const OrderedPredicate &other) const {
+  /// Returns true if this predicate is ordered before `rhs`, based on the cost
+  /// model.
+  bool operator<(const OrderedPredicate &rhs) const {
     // Sort by:
-    // * first and secondary order sums
+    // * higher first and secondary order sums
     // * lower depth
-    // * position dependency
-    // * predicate dependency.
-    auto *otherPos = other.position;
-    return std::make_tuple(other.primary, other.secondary,
-                           otherPos->getIndex().size(), otherPos->getKind(),
-                           other.question->getKind()) >
-           std::make_tuple(primary, secondary, position->getIndex().size(),
-                           position->getKind(), question->getKind());
+    // * lower position dependency
+    // * lower predicate dependency
+    auto *rhsPos = rhs.position;
+    return std::make_tuple(primary, secondary, rhsPos->getOperationDepth(),
+                           rhsPos->getKind(), rhs.question->getKind()) >
+           std::make_tuple(rhs.primary, rhs.secondary,
+                           position->getOperationDepth(), position->getKind(),
+                           question->getKind());
   }
 };
 
