@@ -11,98 +11,90 @@
 //===----------------------------------------------------------------------===//
 
 #include "PassDetail.h"
-#include "mlir/Dialect/Affine/EDSC/Intrinsics.h"
-#include "mlir/Dialect/Linalg/EDSC/FoldedIntrinsics.h"
+#include "mlir/Dialect/Complex/IR/Complex.h"
 #include "mlir/Dialect/Linalg/IR/LinalgOps.h"
 #include "mlir/Dialect/Linalg/IR/LinalgTypes.h"
 #include "mlir/Dialect/Linalg/Passes.h"
 #include "mlir/Dialect/Linalg/Transforms/Transforms.h"
 #include "mlir/Dialect/Linalg/Utils/Utils.h"
 #include "mlir/Dialect/SCF/SCF.h"
-#include "mlir/Dialect/StandardOps/EDSC/Intrinsics.h"
 #include "mlir/IR/AffineExpr.h"
 #include "mlir/IR/AffineExprVisitor.h"
 #include "mlir/IR/AffineMap.h"
+#include "mlir/IR/ImplicitLocOpBuilder.h"
 #include "mlir/Support/LLVM.h"
 #include "mlir/Transforms/FoldUtils.h"
 #include "llvm/ADT/MapVector.h"
+#include "llvm/ADT/TypeSwitch.h"
 #include "llvm/Support/CommandLine.h"
 
 using namespace mlir;
-using namespace mlir::edsc;
-using namespace mlir::edsc::intrinsics;
 using namespace mlir::linalg;
 using namespace mlir::scf;
 
 using llvm::MapVector;
 
-using folded_affine_min = FoldedValueBuilder<AffineMinOp>;
-using folded_linalg_range = FoldedValueBuilder<linalg::RangeOp>;
-using folded_std_dim = FoldedValueBuilder<DimOp>;
-using folded_std_subview = FoldedValueBuilder<SubViewOp>;
-using folded_std_view = FoldedValueBuilder<ViewOp>;
-
 #define DEBUG_TYPE "linalg-promotion"
 
-/// Alloc a new buffer of `size`. If `dynamicBuffers` is true allocate exactly
-/// the size needed, otherwise try to allocate a static bounding box.
-static Value allocBuffer(const LinalgPromotionOptions &options,
-                         Type elementType, Value size, bool dynamicBuffers,
-                         OperationFolder *folder,
+/// Alloc a new buffer of `size` * `width` i8; where `width` is given by the
+/// data `layout` for `elementType`.
+/// Use AllocOp or AllocaOp depending on `options`.
+/// Take an optional alignment.
+static Value allocBuffer(ImplicitLocOpBuilder &b,
+                         const LinalgPromotionOptions &options,
+                         Type elementType, Value allocSize, DataLayout &layout,
                          Optional<unsigned> alignment = None) {
-  auto *ctx = size.getContext();
-  auto width = llvm::divideCeil(elementType.getIntOrFloatBitWidth(), 8);
-  IntegerAttr alignment_attr;
+  auto width = layout.getTypeSize(elementType);
+
+  IntegerAttr alignmentAttr;
   if (alignment.hasValue())
-    alignment_attr =
-        IntegerAttr::get(IntegerType::get(ctx, 64), alignment.getValue());
-  if (!dynamicBuffers)
-    if (auto cst = size.getDefiningOp<ConstantIndexOp>())
-      return options.useAlloca
-                 ? std_alloca(MemRefType::get(width * cst.getValue(),
-                                              IntegerType::get(ctx, 8)),
-                              ValueRange{}, alignment_attr)
-                       .value
-                 : std_alloc(MemRefType::get(width * cst.getValue(),
-                                             IntegerType::get(ctx, 8)),
-                             ValueRange{}, alignment_attr)
-                       .value;
+    alignmentAttr = b.getI64IntegerAttr(alignment.getValue());
+
+  // Static buffer.
+  if (auto cst = allocSize.getDefiningOp<ConstantIndexOp>()) {
+    auto staticBufferType =
+        MemRefType::get(width * cst.getValue(), b.getIntegerType(8));
+    if (options.useAlloca) {
+      return b.createOrFold<memref::AllocaOp>(staticBufferType, ValueRange{},
+                                              alignmentAttr);
+    }
+    return b.createOrFold<memref::AllocOp>(staticBufferType, ValueRange{},
+                                           alignmentAttr);
+  }
+
+  // Fallback dynamic buffer.
+  auto dynamicBufferType = MemRefType::get(-1, b.getIntegerType(8));
   Value mul =
-      folded_std_muli(folder, folded_std_constant_index(folder, width), size);
-  return options.useAlloca
-             ? std_alloca(MemRefType::get(-1, IntegerType::get(ctx, 8)), mul,
-                          alignment_attr)
-                   .value
-             : std_alloc(MemRefType::get(-1, IntegerType::get(ctx, 8)), mul,
-                         alignment_attr)
-                   .value;
+      b.createOrFold<MulIOp>(b.create<ConstantIndexOp>(width), allocSize);
+  if (options.useAlloca)
+    return b.create<memref::AllocaOp>(dynamicBufferType, mul, alignmentAttr);
+  return b.create<memref::AllocOp>(dynamicBufferType, mul, alignmentAttr);
 }
 
 /// Default allocation callback function. This allocates a promoted buffer when
 /// no call back to do so is provided. The default is to allocate a
 /// memref<..xi8> and return a view to get a memref type of shape
 /// boundingSubViewSize.
-static Optional<Value> defaultAllocBufferCallBack(
-    const LinalgPromotionOptions &options, OpBuilder &builder,
-    SubViewOp subView, ArrayRef<Value> boundingSubViewSize, bool dynamicBuffers,
-    Optional<unsigned> alignment, OperationFolder *folder) {
+static Optional<Value>
+defaultAllocBufferCallBack(const LinalgPromotionOptions &options,
+                           OpBuilder &builder, memref::SubViewOp subView,
+                           ArrayRef<Value> boundingSubViewSize,
+                           Optional<unsigned> alignment, DataLayout &layout) {
   ShapedType viewType = subView.getType();
-  int64_t rank = viewType.getRank();
-  (void)rank;
-  assert(rank > 0 && boundingSubViewSize.size() == static_cast<size_t>(rank));
-  auto zero = folded_std_constant_index(folder, 0);
-  auto one = folded_std_constant_index(folder, 1);
+  ImplicitLocOpBuilder b(subView.getLoc(), builder);
+  auto zero = b.createOrFold<ConstantIndexOp>(0);
+  auto one = b.createOrFold<ConstantIndexOp>(1);
 
   Value allocSize = one;
   for (auto size : llvm::enumerate(boundingSubViewSize))
-    allocSize = folded_std_muli(folder, allocSize, size.value());
-  Value buffer = allocBuffer(options, viewType.getElementType(), allocSize,
-                             dynamicBuffers, folder, alignment);
+    allocSize = b.createOrFold<MulIOp>(allocSize, size.value());
+  Value buffer = allocBuffer(b, options, viewType.getElementType(), allocSize,
+                             layout, alignment);
   SmallVector<int64_t, 4> dynSizes(boundingSubViewSize.size(),
                                    ShapedType::kDynamicSize);
-  Value view = folded_std_view(
-      folder, MemRefType::get(dynSizes, viewType.getElementType()), buffer,
-      zero, boundingSubViewSize);
+  Value view = b.createOrFold<memref::ViewOp>(
+      MemRefType::get(dynSizes, viewType.getElementType()), buffer, zero,
+      boundingSubViewSize);
   return view;
 }
 
@@ -112,10 +104,10 @@ static Optional<Value> defaultAllocBufferCallBack(
 static LogicalResult
 defaultDeallocBufferCallBack(const LinalgPromotionOptions &options,
                              OpBuilder &b, Value fullLocalView) {
-  auto viewOp = fullLocalView.getDefiningOp<ViewOp>();
-  assert(viewOp && "expected full local view to be a ViewOp");
-  if (!options.useAlloca)
-    std_dealloc(viewOp.source());
+  if (!options.useAlloca) {
+    auto viewOp = cast<memref::ViewOp>(fullLocalView.getDefiningOp());
+    b.create<memref::DeallocOp>(viewOp.source().getLoc(), viewOp.source());
+  }
   return success();
 }
 
@@ -129,7 +121,7 @@ struct LinalgOpInstancePromotionOptions {
   LinalgOpInstancePromotionOptions(LinalgOp op,
                                    const LinalgPromotionOptions &options);
   /// SubViews to promote.
-  MapVector<unsigned, Value> subViews;
+  MapVector<int64_t, Value> subViews;
   /// True if the full view should be used for the promoted buffer.
   DenseMap<Value, bool> useFullTileBuffers;
 
@@ -142,6 +134,7 @@ struct LinalgOpInstancePromotionOptions {
 
   /// Allow the use of dynamically-sized buffers.
   bool dynamicBuffers;
+
   /// Alignment of promoted buffer.
   Optional<unsigned> alignment;
 };
@@ -152,39 +145,47 @@ LinalgOpInstancePromotionOptions::LinalgOpInstancePromotionOptions(
     : subViews(), dynamicBuffers(options.dynamicBuffers),
       alignment(options.alignment) {
   assert(linalgOp.hasBufferSemantics() && "revisit usage of shaped operand");
-  unsigned nBuffers = linalgOp.getNumShapedOperands();
   auto vUseFullTileBuffers =
       options.useFullTileBuffers.getValueOr(llvm::SmallBitVector());
-  vUseFullTileBuffers.resize(nBuffers, options.useFullTileBuffersDefault);
+  vUseFullTileBuffers.resize(linalgOp.getNumInputsAndOutputs(),
+                             options.useFullTileBuffersDefault);
 
-  for (unsigned idx = 0; idx != nBuffers; ++idx) {
-    if (options.operandsToPromote && !options.operandsToPromote->count(idx))
+  for (OpOperand *opOperand : linalgOp.getInputAndOutputOperands()) {
+    int64_t operandNumber = opOperand->getOperandNumber();
+    if (options.operandsToPromote &&
+        !options.operandsToPromote->count(operandNumber))
       continue;
-    auto *op = linalgOp.getShapedOperand(idx).getDefiningOp();
-    if (auto sv = dyn_cast_or_null<SubViewOp>(op)) {
-      subViews[idx] = sv;
-      useFullTileBuffers[sv] = vUseFullTileBuffers[idx];
+    Operation *op = opOperand->get().getDefiningOp();
+    if (auto sv = dyn_cast_or_null<memref::SubViewOp>(op)) {
+      subViews[operandNumber] = sv;
+      useFullTileBuffers[sv] = vUseFullTileBuffers[operandNumber];
     }
   }
 
-  allocationFn =
-      (options.allocationFn ? *(options.allocationFn)
-                            : [&](OpBuilder &builder, SubViewOp subViewOp,
-                                  ArrayRef<Value> boundingSubViewSize,
-                                  OperationFolder *folder) -> Optional<Value> {
-        return defaultAllocBufferCallBack(options, builder, subViewOp,
-                                          boundingSubViewSize, dynamicBuffers,
-                                          alignment, folder);
-      });
-  deallocationFn =
-      (options.deallocationFn
-           ? *(options.deallocationFn)
-           : [&](OpBuilder &b, Value buffer) {
-               return defaultDeallocBufferCallBack(options, b, buffer);
-             });
-  auto defaultCopyCallBack = [&](OpBuilder &builder, Value src,
-                                 Value dst) -> LogicalResult {
-    linalg_copy(src, dst);
+  if (options.allocationFn) {
+    allocationFn = *options.allocationFn;
+  } else {
+    allocationFn = [&](OpBuilder &b, memref::SubViewOp subViewOp,
+                       ArrayRef<Value> boundingSubViewSize,
+                       DataLayout &layout) -> Optional<Value> {
+      return defaultAllocBufferCallBack(options, b, subViewOp,
+                                        boundingSubViewSize, alignment, layout);
+    };
+  }
+
+  if (options.deallocationFn) {
+    deallocationFn = *options.deallocationFn;
+  } else {
+    deallocationFn = [&](OpBuilder &b, Value buffer) {
+      return defaultDeallocBufferCallBack(options, b, buffer);
+    };
+  }
+
+  // Save the loc because `linalgOp` goes out of scope.
+  Location loc = linalgOp.getLoc();
+  auto defaultCopyCallBack = [loc](OpBuilder &b, Value src,
+                                   Value dst) -> LogicalResult {
+    b.create<linalg::CopyOp>(loc, src, dst);
     return success();
   };
   copyInFn = (options.copyInFn ? *(options.copyInFn) : defaultCopyCallBack);
@@ -209,9 +210,8 @@ LinalgOpInstancePromotionOptions::LinalgOpInstancePromotionOptions(
 // boundary tiles. For now this is done with an unconditional `fill` op followed
 // by a partial `copy` op.
 Optional<PromotionInfo> mlir::linalg::promoteSubviewAsNewBuffer(
-    OpBuilder &b, Location loc, SubViewOp subView,
-    AllocBufferCallbackFn allocationFn, OperationFolder *folder) {
-  ScopedContext scopedContext(b, loc);
+    OpBuilder &b, Location loc, memref::SubViewOp subView,
+    AllocBufferCallbackFn allocationFn, DataLayout &layout) {
   auto viewType = subView.getType();
   auto rank = viewType.getRank();
   SmallVector<Value, 4> fullSizes;
@@ -227,35 +227,35 @@ Optional<PromotionInfo> mlir::linalg::promoteSubviewAsNewBuffer(
         (!sizeAttr) ? rangeValue.size : b.create<ConstantOp>(loc, sizeAttr);
     LLVM_DEBUG(llvm::dbgs() << "Extracted tightest: " << size << "\n");
     fullSizes.push_back(size);
-    partialSizes.push_back(folded_std_dim(folder, subView, en.index()).value);
+    partialSizes.push_back(
+        b.createOrFold<memref::DimOp>(loc, subView, en.index()));
   }
   SmallVector<int64_t, 4> dynSizes(fullSizes.size(), -1);
   // If a callback is not specified, then use the default implementation for
   // allocating the promoted buffer.
-  Optional<Value> fullLocalView = allocationFn(b, subView, fullSizes, folder);
+  Optional<Value> fullLocalView = allocationFn(b, subView, fullSizes, layout);
   if (!fullLocalView)
     return {};
   SmallVector<OpFoldResult, 4> zeros(fullSizes.size(), b.getIndexAttr(0));
   SmallVector<OpFoldResult, 4> ones(fullSizes.size(), b.getIndexAttr(1));
-  auto partialLocalView =
-      folded_std_subview(folder, *fullLocalView, zeros, partialSizes, ones);
+  auto partialLocalView = b.createOrFold<memref::SubViewOp>(
+      loc, *fullLocalView, zeros, partialSizes, ones);
   return PromotionInfo{*fullLocalView, partialLocalView};
 }
 
-static Optional<MapVector<unsigned, PromotionInfo>>
-promoteSubViews(OpBuilder &b, Location loc,
-                LinalgOpInstancePromotionOptions options,
-                OperationFolder *folder) {
+static Optional<MapVector<int64_t, PromotionInfo>>
+promoteSubViews(ImplicitLocOpBuilder &b,
+                LinalgOpInstancePromotionOptions options, DataLayout &layout) {
   if (options.subViews.empty())
     return {};
 
-  ScopedContext scope(b, loc);
-  MapVector<unsigned, PromotionInfo> promotionInfoMap;
+  MapVector<int64_t, PromotionInfo> promotionInfoMap;
 
   for (auto v : options.subViews) {
-    SubViewOp subView = cast<SubViewOp>(v.second.getDefiningOp());
+    memref::SubViewOp subView =
+        cast<memref::SubViewOp>(v.second.getDefiningOp());
     Optional<PromotionInfo> promotionInfo = promoteSubviewAsNewBuffer(
-        b, loc, subView, options.allocationFn, folder);
+        b, b.getLoc(), subView, options.allocationFn, layout);
     if (!promotionInfo)
       return {};
     promotionInfoMap[v.first] = *promotionInfo;
@@ -263,13 +263,27 @@ promoteSubViews(OpBuilder &b, Location loc,
     // Only fill the buffer if the full local view is used
     if (!options.useFullTileBuffers[v.second])
       continue;
-    Value fillVal;
-    if (auto t = subView.getType().getElementType().dyn_cast<FloatType>())
-      fillVal = folded_std_constant(folder, FloatAttr::get(t, 0.0));
-    else if (auto t =
-                 subView.getType().getElementType().dyn_cast<IntegerType>())
-      fillVal = folded_std_constant_int(folder, 0, t);
-    linalg_fill(promotionInfo->fullLocalView, fillVal);
+    Type subviewEltType = subView.getType().getElementType();
+    Value fillVal =
+        llvm::TypeSwitch<Type, Value>(subviewEltType)
+            .Case([&](FloatType t) {
+              return b.create<ConstantOp>(FloatAttr::get(t, 0.0));
+            })
+            .Case([&](IntegerType t) {
+              return b.create<ConstantOp>(IntegerAttr::get(t, 0));
+            })
+            .Case([&](ComplexType t) {
+              Value tmp;
+              if (auto et = t.getElementType().dyn_cast<FloatType>())
+                tmp = b.create<ConstantOp>(FloatAttr::get(et, 0.0));
+              else if (auto et = t.getElementType().cast<IntegerType>())
+                tmp = b.create<ConstantOp>(IntegerAttr::get(et, 0));
+              return b.create<complex::CreateOp>(t, tmp, tmp);
+            })
+            .Default([](auto) { return Value(); });
+    if (!fillVal)
+      return {};
+    b.create<linalg::FillOp>(fillVal, promotionInfo->fullLocalView);
   }
 
   // Copy data into the promoted buffers. Use callback if provided.
@@ -277,17 +291,17 @@ promoteSubViews(OpBuilder &b, Location loc,
     auto info = promotionInfoMap.find(v.first);
     if (info == promotionInfoMap.end())
       continue;
-    if (failed(options.copyInFn(b, cast<SubViewOp>(v.second.getDefiningOp()),
-                                info->second.partialLocalView)))
+    if (failed(options.copyInFn(
+            b, cast<memref::SubViewOp>(v.second.getDefiningOp()),
+            info->second.partialLocalView)))
       return {};
   }
   return promotionInfoMap;
 }
 
 static Optional<LinalgOp>
-promoteSubViews(OpBuilder &b, LinalgOp op,
-                LinalgOpInstancePromotionOptions options,
-                OperationFolder *folder) {
+promoteSubViews(ImplicitLocOpBuilder &b, LinalgOp op,
+                LinalgOpInstancePromotionOptions options, DataLayout &layout) {
   assert(op.hasBufferSemantics() && "expected linalg op with buffer semantics");
 
   if (auto convOp = dyn_cast<linalg::ConvOp>(op.getOperation())) {
@@ -297,8 +311,7 @@ promoteSubViews(OpBuilder &b, LinalgOp op,
   }
 
   // 1. Promote the specified views and use them in the new op.
-  auto loc = op.getLoc();
-  auto promotedBuffersAndViews = promoteSubViews(b, loc, options, folder);
+  auto promotedBuffersAndViews = promoteSubViews(b, options, layout);
   if (!promotedBuffersAndViews ||
       promotedBuffersAndViews->size() != options.subViews.size())
     return {};
@@ -307,30 +320,30 @@ promoteSubViews(OpBuilder &b, LinalgOp op,
   // operands are not views. This is to support cases such as FillOp taking
   // extra scalars etc.  Keep a reference to output buffers;
   SmallVector<Value, 8> opViews;
-  opViews.reserve(op.getNumShapedOperands());
+  opViews.reserve(op.getNumInputsAndOutputs());
   SmallVector<std::pair<Value, Value>, 8> writebackViews;
   writebackViews.reserve(promotedBuffersAndViews->size());
-  for (auto view : llvm::enumerate(op.getShapedOperands())) {
-    if (options.subViews.count(view.index()) != 0) {
-      if (options.useFullTileBuffers[view.value()])
+  for (OpOperand *opOperand : op.getInputAndOutputOperands()) {
+    int64_t operandNumber = opOperand->getOperandNumber();
+    if (options.subViews.count(operandNumber) != 0) {
+      if (options.useFullTileBuffers[opOperand->get()])
         opViews.push_back(
-            (*promotedBuffersAndViews)[view.index()].fullLocalView);
+            (*promotedBuffersAndViews)[operandNumber].fullLocalView);
       else
         opViews.push_back(
-            (*promotedBuffersAndViews)[view.index()].partialLocalView);
-      if (view.index() >= op.getNumInputs())
+            (*promotedBuffersAndViews)[operandNumber].partialLocalView);
+      if (operandNumber >= op.getNumInputs())
         writebackViews.emplace_back(std::make_pair(
-            view.value(),
-            (*promotedBuffersAndViews)[view.index()].partialLocalView));
+            opOperand->get(),
+            (*promotedBuffersAndViews)[operandNumber].partialLocalView));
     } else {
-      opViews.push_back(view.value());
+      opViews.push_back(opOperand->get());
     }
   }
   op->setOperands(0, opViews.size(), opViews);
 
   OpBuilder::InsertionGuard guard(b);
   b.setInsertionPointAfter(op);
-  ScopedContext scope(b, loc);
   // 3. Emit write-back for the promoted output views: copy the partial view.
   for (auto viewAndPartialLocalView : writebackViews) {
     if (failed(options.copyOutFn(b, viewAndPartialLocalView.second,
@@ -340,23 +353,24 @@ promoteSubViews(OpBuilder &b, LinalgOp op,
 
   // 4. Dealloc all local buffers.
   for (const auto &pi : *promotedBuffersAndViews)
-    options.deallocationFn(b, pi.second.fullLocalView);
+    (void)options.deallocationFn(b, pi.second.fullLocalView);
   return op;
 }
 
 LogicalResult
 mlir::linalg::promoteSubviewsPrecondition(Operation *op,
                                           LinalgPromotionOptions options) {
-  LinalgOp linOp = dyn_cast<LinalgOp>(op);
+  LinalgOp linalgOp = dyn_cast<LinalgOp>(op);
   // Transformation applies to buffers only.
-  if (!linOp || !linOp.hasBufferSemantics())
+  if (!linalgOp || !linalgOp.hasBufferSemantics())
     return failure();
   // Check that at least one of the requested operands is indeed a subview.
-  for (auto en : llvm::enumerate(linOp.getShapedOperands())) {
-    auto sv = isa_and_nonnull<SubViewOp>(en.value().getDefiningOp());
+  for (OpOperand *opOperand : linalgOp.getInputAndOutputOperands()) {
+    auto sv =
+        isa_and_nonnull<memref::SubViewOp>(opOperand->get().getDefiningOp());
     if (sv) {
       if (!options.operandsToPromote.hasValue() ||
-          options.operandsToPromote->count(en.index()))
+          options.operandsToPromote->count(opOperand->getOperandNumber()))
         return success();
     }
   }
@@ -365,13 +379,13 @@ mlir::linalg::promoteSubviewsPrecondition(Operation *op,
   return failure();
 }
 
-Optional<LinalgOp> mlir::linalg::promoteSubViews(OpBuilder &b,
-                                                 LinalgOp linalgOp,
-                                                 LinalgPromotionOptions options,
-                                                 OperationFolder *folder) {
+Optional<LinalgOp>
+mlir::linalg::promoteSubViews(OpBuilder &builder, LinalgOp linalgOp,
+                              LinalgPromotionOptions options) {
   LinalgOpInstancePromotionOptions linalgOptions(linalgOp, options);
-  return ::promoteSubViews(
-      b, linalgOp, LinalgOpInstancePromotionOptions(linalgOp, options), folder);
+  auto layout = DataLayout::closest(linalgOp);
+  ImplicitLocOpBuilder b(linalgOp.getLoc(), builder);
+  return ::promoteSubViews(b, linalgOp, linalgOptions, layout);
 }
 
 namespace {
@@ -383,16 +397,15 @@ struct LinalgPromotionPass : public LinalgPromotionBase<LinalgPromotionPass> {
   }
 
   void runOnFunction() override {
-    OperationFolder folder(&getContext());
-    getFunction().walk([this, &folder](LinalgOp op) {
+    getFunction().walk([&](LinalgOp op) {
       auto options = LinalgPromotionOptions()
                          .setDynamicBuffers(dynamicBuffers)
                          .setUseAlloca(useAlloca);
       if (failed(promoteSubviewsPrecondition(op, options)))
         return;
       LLVM_DEBUG(llvm::dbgs() << "Promote: " << *(op.getOperation()) << "\n");
-      OpBuilder b(op);
-      promoteSubViews(b, op, options, &folder);
+      ImplicitLocOpBuilder b(op.getLoc(), op);
+      promoteSubViews(b, op, options);
     });
   }
 };
