@@ -91,7 +91,7 @@ void mlir::outlineIfOp(OpBuilder &b, scf::IfOp ifOp, FuncOp *thenFn,
     OpBuilder::InsertionGuard g(b);
     b.setInsertionPoint(ifOp->getParentOfType<FuncOp>());
 
-    llvm::SetVector<Value> captures;
+    SetVector<Value> captures;
     getUsedValuesDefinedAbove(ifOrElseRegion, captures);
 
     ValueRange values(captures.getArrayRef());
@@ -145,3 +145,43 @@ bool mlir::getInnermostParallelLoops(Operation *rootOp,
   }
   return rootEnclosesPloops;
 }
+
+/// Given the `lbVal`, `ubVal` and `stepVal` of a loop, append `lbVal` and
+/// `ubVal` to `dims` and `stepVal` to `symbols`.
+/// Create new AffineDimExpr (`%lb` and `%ub`) and AffineSymbolExpr (`%step`)
+/// with positions matching the newly appended values. Then create a min
+/// expression (i.e. `%lb`) and a max expression
+/// (i.e. `%lb + %step * floordiv(%ub -1 - %lb, %step)`.
+static std::pair<AffineExpr, AffineExpr>
+getMinMaxLoopIndVar(Value lbVal, Value ubVal, Value stepVal,
+                    SmallVectorImpl<Value> &dims,
+                    SmallVectorImpl<Value> &symbols) {
+  MLIRContext *ctx = lbVal.getContext();
+  AffineExpr lb = getAffineDimExpr(dims.size(), ctx);
+  dims.push_back(lbVal);
+  AffineExpr ub = getAffineDimExpr(dims.size(), ctx);
+  dims.push_back(ubVal);
+  AffineExpr step = getAffineSymbolExpr(symbols.size(), ctx);
+  symbols.push_back(stepVal);
+  return std::make_pair(lb, lb + step * ((ub - 1) - lb).floorDiv(step));
+}
+
+/// Return the min/max expressions for `value` if it is an induction variable
+/// from scf.for or scf.parallel loop.
+/// if `loopFilter` is passed, the filter determines which loop to consider.
+/// Other induction variables are ignored.
+Optional<std::pair<AffineExpr, AffineExpr>> mlir::getSCFMinMaxExpr(
+    Value value, SmallVectorImpl<Value> &dims, SmallVectorImpl<Value> &symbols,
+    llvm::function_ref<bool(Operation *)> substituteOperation) {
+  if (auto forOp = scf::getForInductionVarOwner(value))
+    return getMinMaxLoopIndVar(forOp.lowerBound(), forOp.upperBound(),
+                               forOp.step(), dims, symbols);
+
+  if (auto parallelForOp = scf::getParallelForInductionVarOwner(value))
+    for (unsigned idx = 0, e = parallelForOp.getNumLoops(); idx < e; ++idx)
+      if (parallelForOp.getInductionVars()[idx] == value)
+        return getMinMaxLoopIndVar(parallelForOp.lowerBound()[idx],
+                                   parallelForOp.upperBound()[idx],
+                                   parallelForOp.step()[idx], dims, symbols);
+  return {};
+}
