@@ -12,6 +12,8 @@
 using namespace mlir;
 using namespace mlir::tensor;
 
+#include "mlir/Dialect/Tensor/IR/TensorOpsDialect.cpp.inc"
+
 //===----------------------------------------------------------------------===//
 // TensorDialect Dialect Interfaces
 //===----------------------------------------------------------------------===//
@@ -30,6 +32,10 @@ struct TensorInlinerInterface : public DialectInlinerInterface {
 };
 } // end anonymous namespace
 
+//===----------------------------------------------------------------------===//
+// TensorDialect Methods
+//===----------------------------------------------------------------------===//
+
 void TensorDialect::initialize() {
   addOperations<
 #define GET_OP_LIST
