@@ -14,7 +14,7 @@
 #include "mlir/Dialect/LLVMIR/LLVMDialect.h"
 #include "mlir/IR/BuiltinOps.h"
 #include "mlir/Support/FileUtilities.h"
-#include "mlir/Target/LLVMIR.h"
+#include "mlir/Target/LLVMIR/Export.h"
 
 #include "llvm/ExecutionEngine/JITEventListener.h"
 #include "llvm/ExecutionEngine/ObjectCache.h"
@@ -179,11 +179,13 @@ static void packFunctionArguments(Module *module) {
     for (auto &indexedArg : llvm::enumerate(func.args())) {
       llvm::Value *argIndex = llvm::Constant::getIntegerValue(
           builder.getInt64Ty(), APInt(64, indexedArg.index()));
-      llvm::Value *argPtrPtr = builder.CreateGEP(argList, argIndex);
-      llvm::Value *argPtr = builder.CreateLoad(argPtrPtr);
-      argPtr = builder.CreateBitCast(
-          argPtr, indexedArg.value().getType()->getPointerTo());
-      llvm::Value *arg = builder.CreateLoad(argPtr);
+      llvm::Value *argPtrPtr = builder.CreateGEP(
+          builder.getInt8PtrTy(), argList, argIndex);
+      llvm::Value *argPtr = builder.CreateLoad(builder.getInt8PtrTy(),
+                                               argPtrPtr);
+      llvm::Type *argTy = indexedArg.value().getType();
+      argPtr = builder.CreateBitCast(argPtr, argTy->getPointerTo());
+      llvm::Value *arg = builder.CreateLoad(argTy, argPtr);
       args.push_back(arg);
     }
 
@@ -194,8 +196,10 @@ static void packFunctionArguments(Module *module) {
     if (!result->getType()->isVoidTy()) {
       llvm::Value *retIndex = llvm::Constant::getIntegerValue(
           builder.getInt64Ty(), APInt(64, llvm::size(func.args())));
-      llvm::Value *retPtrPtr = builder.CreateGEP(argList, retIndex);
-      llvm::Value *retPtr = builder.CreateLoad(retPtrPtr);
+      llvm::Value *retPtrPtr =
+          builder.CreateGEP(builder.getInt8PtrTy(), argList, retIndex);
+      llvm::Value *retPtr = builder.CreateLoad(builder.getInt8PtrTy(),
+                                               retPtrPtr);
       retPtr = builder.CreateBitCast(retPtr, result->getType()->getPointerTo());
       builder.CreateStore(result, retPtr);
     }
@@ -255,6 +259,15 @@ Expected<std::unique_ptr<ExecutionEngine>> ExecutionEngine::create(
     if (engine->perfListener)
       objectLayer->registerJITEventListener(*engine->perfListener);
 
+    // COFF format binaries (Windows) need special handling to deal with
+    // exported symbol visibility.
+    // cf llvm/lib/ExecutionEngine/Orc/LLJIT.cpp LLJIT::createObjectLinkingLayer
+    llvm::Triple targetTriple(llvm::Twine(llvmModule->getTargetTriple()));
+    if (targetTriple.isOSBinFormatCOFF()) {
+      objectLayer->setOverrideObjectFlagsWithResponsibilityFlags(true);
+      objectLayer->setAutoClaimResponsibilityForObjectSymbols(true);
+    }
+
     // Resolve symbols from shared libraries.
     for (auto libPath : sharedLibPaths) {
       auto mb = llvm::MemoryBuffer::getFile(libPath);
@@ -339,7 +352,8 @@ Expected<void (*)(void **)> ExecutionEngine::lookup(StringRef name) const {
   return fptr;
 }
 
-Error ExecutionEngine::invoke(StringRef name, MutableArrayRef<void *> args) {
+Error ExecutionEngine::invokePacked(StringRef name,
+                                    MutableArrayRef<void *> args) {
   auto expectedFPtr = lookup(name);
   if (!expectedFPtr)
     return expectedFPtr.takeError();
