@@ -12,6 +12,8 @@
 #include "mlir/Pass/Pass.h"
 
 namespace mlir {
+class AffineDialect;
+
 // Forward declaration from Dialect.h
 template <typename ConcreteDialect>
 void registerDialect(DialectRegistry &registry);
@@ -20,6 +22,10 @@ namespace linalg {
 class LinalgDialect;
 } // end namespace linalg
 
+namespace memref {
+class MemRefDialect;
+} // end namespace memref
+
 #define GEN_PASS_CLASSES
 #include "mlir/Transforms/Passes.h.inc"
 
