@@ -23,6 +23,10 @@ namespace {
 struct PrintOpAvailability
     : public PassWrapper<PrintOpAvailability, FunctionPass> {
   void runOnFunction() override;
+  StringRef getArgument() const final { return "test-spirv-op-availability"; }
+  StringRef getDescription() const final {
+    return "Test SPIR-V op availability";
+  }
 };
 } // end anonymous namespace
 
@@ -78,8 +82,7 @@ void PrintOpAvailability::runOnFunction() {
 
 namespace mlir {
 void registerPrintOpAvailabilityPass() {
-  PassRegistration<PrintOpAvailability> printOpAvailabilityPass(
-      "test-spirv-op-availability", "Test SPIR-V op availability");
+  PassRegistration<PrintOpAvailability>();
 }
 } // namespace mlir
 
@@ -91,6 +94,10 @@ namespace {
 /// A pass for testing SPIR-V op availability.
 struct ConvertToTargetEnv
     : public PassWrapper<ConvertToTargetEnv, FunctionPass> {
+  StringRef getArgument() const override { return "test-spirv-target-env"; }
+  StringRef getDescription() const override {
+    return "Test SPIR-V target environment";
+  }
   void runOnFunction() override;
 };
 
@@ -137,20 +144,20 @@ void ConvertToTargetEnv::runOnFunction() {
     return signalPassFailure();
   }
 
-  auto target = spirv::SPIRVConversionTarget::get(targetEnv);
+  auto target = SPIRVConversionTarget::get(targetEnv);
 
-  OwningRewritePatternList patterns;
-  patterns.insert<ConvertToAtomCmpExchangeWeak, ConvertToBitReverse,
-                  ConvertToGroupNonUniformBallot, ConvertToModule,
-                  ConvertToSubgroupBallot>(context);
+  RewritePatternSet patterns(context);
+  patterns.add<ConvertToAtomCmpExchangeWeak, ConvertToBitReverse,
+               ConvertToGroupNonUniformBallot, ConvertToModule,
+               ConvertToSubgroupBallot>(context);
 
   if (failed(applyPartialConversion(fn, *target, std::move(patterns))))
     return signalPassFailure();
 }
 
 ConvertToAtomCmpExchangeWeak::ConvertToAtomCmpExchangeWeak(MLIRContext *context)
-    : RewritePattern("test.convert_to_atomic_compare_exchange_weak_op",
-                     {"spv.AtomicCompareExchangeWeak"}, 1, context) {}
+    : RewritePattern("test.convert_to_atomic_compare_exchange_weak_op", 1,
+                     context, {"spv.AtomicCompareExchangeWeak"}) {}
 
 LogicalResult
 ConvertToAtomCmpExchangeWeak::matchAndRewrite(Operation *op,
@@ -170,8 +177,8 @@ ConvertToAtomCmpExchangeWeak::matchAndRewrite(Operation *op,
 }
 
 ConvertToBitReverse::ConvertToBitReverse(MLIRContext *context)
-    : RewritePattern("test.convert_to_bit_reverse_op", {"spv.BitReverse"}, 1,
-                     context) {}
+    : RewritePattern("test.convert_to_bit_reverse_op", 1, context,
+                     {"spv.BitReverse"}) {}
 
 LogicalResult
 ConvertToBitReverse::matchAndRewrite(Operation *op,
@@ -185,8 +192,8 @@ ConvertToBitReverse::matchAndRewrite(Operation *op,
 
 ConvertToGroupNonUniformBallot::ConvertToGroupNonUniformBallot(
     MLIRContext *context)
-    : RewritePattern("test.convert_to_group_non_uniform_ballot_op",
-                     {"spv.GroupNonUniformBallot"}, 1, context) {}
+    : RewritePattern("test.convert_to_group_non_uniform_ballot_op", 1, context,
+                     {"spv.GroupNonUniformBallot"}) {}
 
 LogicalResult ConvertToGroupNonUniformBallot::matchAndRewrite(
     Operation *op, PatternRewriter &rewriter) const {
@@ -198,7 +205,7 @@ LogicalResult ConvertToGroupNonUniformBallot::matchAndRewrite(
 }
 
 ConvertToModule::ConvertToModule(MLIRContext *context)
-    : RewritePattern("test.convert_to_module_op", {"spv.module"}, 1, context) {}
+    : RewritePattern("test.convert_to_module_op", 1, context, {"spv.module"}) {}
 
 LogicalResult
 ConvertToModule::matchAndRewrite(Operation *op,
@@ -210,8 +217,8 @@ ConvertToModule::matchAndRewrite(Operation *op,
 }
 
 ConvertToSubgroupBallot::ConvertToSubgroupBallot(MLIRContext *context)
-    : RewritePattern("test.convert_to_subgroup_ballot_op",
-                     {"spv.SubgroupBallotKHR"}, 1, context) {}
+    : RewritePattern("test.convert_to_subgroup_ballot_op", 1, context,
+                     {"spv.SubgroupBallotKHR"}) {}
 
 LogicalResult
 ConvertToSubgroupBallot::matchAndRewrite(Operation *op,
@@ -225,7 +232,6 @@ ConvertToSubgroupBallot::matchAndRewrite(Operation *op,
 
 namespace mlir {
 void registerConvertToTargetEnvPass() {
-  PassRegistration<ConvertToTargetEnv> convertToTargetEnvPass(
-      "test-spirv-target-env", "Test SPIR-V target environment");
+  PassRegistration<ConvertToTargetEnv>();
 }
 } // namespace mlir
