@@ -17,6 +17,10 @@ namespace {
 /// provided by the symbol table along with erasing from the symbol table.
 struct SymbolUsesPass
     : public PassWrapper<SymbolUsesPass, OperationPass<ModuleOp>> {
+  StringRef getArgument() const final { return "test-symbol-uses"; }
+  StringRef getDescription() const final {
+    return "Test detection of symbol uses";
+  }
   WalkResult operateOnSymbol(Operation *symbol, ModuleOp module,
                              SmallVectorImpl<FuncOp> &deadFunctions) {
     // Test computing uses on a non symboltable op.
@@ -89,17 +93,25 @@ struct SymbolUsesPass
 /// functionality provided by the symbol table.
 struct SymbolReplacementPass
     : public PassWrapper<SymbolReplacementPass, OperationPass<ModuleOp>> {
+  StringRef getArgument() const final { return "test-symbol-rauw"; }
+  StringRef getDescription() const final {
+    return "Test replacement of symbol uses";
+  }
   void runOnOperation() override {
-    auto module = getOperation();
+    ModuleOp module = getOperation();
 
-    // Walk nested functions and modules.
+    // Don't try to replace if we can't collect symbol uses.
+    if (!SymbolTable::getSymbolUses(&module.getBodyRegion()))
+      return;
+
+    SymbolTableCollection symbolTable;
+    SymbolUserMap symbolUsers(symbolTable, module);
     module.getBodyRegion().walk([&](Operation *nestedOp) {
       StringAttr newName = nestedOp->getAttrOfType<StringAttr>("sym.new_name");
       if (!newName)
         return;
-      if (succeeded(SymbolTable::replaceAllSymbolUses(
-              nestedOp, newName.getValue(), &module.getBodyRegion())))
-        SymbolTable::setSymbolName(nestedOp, newName.getValue());
+      symbolUsers.replaceAllUsesWith(nestedOp, newName.getValue());
+      SymbolTable::setSymbolName(nestedOp, newName.getValue());
     });
   }
 };
@@ -107,10 +119,8 @@ struct SymbolReplacementPass
 
 namespace mlir {
 void registerSymbolTestPasses() {
-  PassRegistration<SymbolUsesPass>("test-symbol-uses",
-                                   "Test detection of symbol uses");
+  PassRegistration<SymbolUsesPass>();
 
-  PassRegistration<SymbolReplacementPass>("test-symbol-rauw",
-                                          "Test replacement of symbol uses");
+  PassRegistration<SymbolReplacementPass>();
 }
 } // namespace mlir
